# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _plotstuff_c
else:
    import _plotstuff_c

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import astrometry.util.util
PLOTSTUFF_FORMAT_JPG = _plotstuff_c.PLOTSTUFF_FORMAT_JPG
PLOTSTUFF_FORMAT_PNG = _plotstuff_c.PLOTSTUFF_FORMAT_PNG
PLOTSTUFF_FORMAT_PPM = _plotstuff_c.PLOTSTUFF_FORMAT_PPM
PLOTSTUFF_FORMAT_PDF = _plotstuff_c.PLOTSTUFF_FORMAT_PDF
PLOTSTUFF_FORMAT_MEMIMG = _plotstuff_c.PLOTSTUFF_FORMAT_MEMIMG
PLOTSTUFF_FORMAT_FITS = _plotstuff_c.PLOTSTUFF_FORMAT_FITS
class plot_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    plotters = property(_plotstuff_c.plot_args_plotters_get, _plotstuff_c.plot_args_plotters_set)
    NP = property(_plotstuff_c.plot_args_NP_get, _plotstuff_c.plot_args_NP_set)
    outfn = property(_plotstuff_c.plot_args_outfn_get, _plotstuff_c.plot_args_outfn_set)
    fout = property(_plotstuff_c.plot_args_fout_get, _plotstuff_c.plot_args_fout_set)
    outformat = property(_plotstuff_c.plot_args_outformat_get, _plotstuff_c.plot_args_outformat_set)
    outimage = property(_plotstuff_c.plot_args_outimage_get, _plotstuff_c.plot_args_outimage_set)
    cairo = property(_plotstuff_c.plot_args_cairo_get, _plotstuff_c.plot_args_cairo_set)
    target = property(_plotstuff_c.plot_args_target_get, _plotstuff_c.plot_args_target_set)
    op = property(_plotstuff_c.plot_args_op_get, _plotstuff_c.plot_args_op_set)
    move_to = property(_plotstuff_c.plot_args_move_to_get, _plotstuff_c.plot_args_move_to_set)
    move_to_baton = property(_plotstuff_c.plot_args_move_to_baton_get, _plotstuff_c.plot_args_move_to_baton_set)
    line_to = property(_plotstuff_c.plot_args_line_to_get, _plotstuff_c.plot_args_line_to_set)
    line_to_baton = property(_plotstuff_c.plot_args_line_to_baton_get, _plotstuff_c.plot_args_line_to_baton_set)
    wcs = property(_plotstuff_c.plot_args_wcs_get, _plotstuff_c.plot_args_wcs_set)
    W = property(_plotstuff_c.plot_args_W_get, _plotstuff_c.plot_args_W_set)
    H = property(_plotstuff_c.plot_args_H_get, _plotstuff_c.plot_args_H_set)
    rgba = property(_plotstuff_c.plot_args_rgba_get, _plotstuff_c.plot_args_rgba_set)
    lw = property(_plotstuff_c.plot_args_lw_get, _plotstuff_c.plot_args_lw_set)
    marker = property(_plotstuff_c.plot_args_marker_get, _plotstuff_c.plot_args_marker_set)
    markersize = property(_plotstuff_c.plot_args_markersize_get, _plotstuff_c.plot_args_markersize_set)
    bg_rgba = property(_plotstuff_c.plot_args_bg_rgba_get, _plotstuff_c.plot_args_bg_rgba_set)
    bg_lw = property(_plotstuff_c.plot_args_bg_lw_get, _plotstuff_c.plot_args_bg_lw_set)
    bg_box = property(_plotstuff_c.plot_args_bg_box_get, _plotstuff_c.plot_args_bg_box_set)
    fontsize = property(_plotstuff_c.plot_args_fontsize_get, _plotstuff_c.plot_args_fontsize_set)
    halign = property(_plotstuff_c.plot_args_halign_get, _plotstuff_c.plot_args_halign_set)
    valign = property(_plotstuff_c.plot_args_valign_get, _plotstuff_c.plot_args_valign_set)
    label_offset_x = property(_plotstuff_c.plot_args_label_offset_x_get, _plotstuff_c.plot_args_label_offset_x_set)
    label_offset_y = property(_plotstuff_c.plot_args_label_offset_y_get, _plotstuff_c.plot_args_label_offset_y_set)
    text_bg_layer = property(_plotstuff_c.plot_args_text_bg_layer_get, _plotstuff_c.plot_args_text_bg_layer_set)
    text_fg_layer = property(_plotstuff_c.plot_args_text_fg_layer_get, _plotstuff_c.plot_args_text_fg_layer_set)
    marker_fg_layer = property(_plotstuff_c.plot_args_marker_fg_layer_get, _plotstuff_c.plot_args_marker_fg_layer_set)
    cairocmds = property(_plotstuff_c.plot_args_cairocmds_get, _plotstuff_c.plot_args_cairocmds_set)
    linestep = property(_plotstuff_c.plot_args_linestep_get, _plotstuff_c.plot_args_linestep_set)

    def view_image_as_numpy(self):
        return _plotstuff_c.plot_args_view_image_as_numpy(self)

    def get_image_as_numpy(self, flip, out):
        return _plotstuff_c.plot_args_get_image_as_numpy(self, flip, out)

    def get_image_as_numpy_view(self):
        return _plotstuff_c.plot_args_get_image_as_numpy_view(self)

    def set_image_from_numpy(self, py_img, flip):
        return _plotstuff_c.plot_args_set_image_from_numpy(self, py_img, flip)

    def set_wcs_file(self, fn, ext):
        return _plotstuff_c.plot_args_set_wcs_file(self, fn, ext)

    def set_size_from_wcs(self):
        return _plotstuff_c.plot_args_set_size_from_wcs(self)

    def loginit(self, level):
        return _plotstuff_c.plot_args_loginit(self, level)

    def __init__(self):
        _plotstuff_c.plot_args_swiginit(self, _plotstuff_c.new_plot_args())
    __swig_destroy__ = _plotstuff_c.delete_plot_args

# Register plot_args in _plotstuff_c:
_plotstuff_c.plot_args_swigregister(plot_args)

class plotter(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    name = property(_plotstuff_c.plotter_name_get, _plotstuff_c.plotter_name_set)
    init = property(_plotstuff_c.plotter_init_get, _plotstuff_c.plotter_init_set)
    init2 = property(_plotstuff_c.plotter_init2_get, _plotstuff_c.plotter_init2_set)
    command = property(_plotstuff_c.plotter_command_get, _plotstuff_c.plotter_command_set)
    doplot = property(_plotstuff_c.plotter_doplot_get, _plotstuff_c.plotter_doplot_set)
    free = property(_plotstuff_c.plotter_free_get, _plotstuff_c.plotter_free_set)
    baton = property(_plotstuff_c.plotter_baton_get, _plotstuff_c.plotter_baton_set)

    def __init__(self):
        _plotstuff_c.plotter_swiginit(self, _plotstuff_c.new_plotter())
    __swig_destroy__ = _plotstuff_c.delete_plotter

# Register plotter in _plotstuff_c:
_plotstuff_c.plotter_swigregister(plotter)


def parse_image_format(fmt):
    return _plotstuff_c.parse_image_format(fmt)

def guess_image_format_from_filename(fn):
    return _plotstuff_c.guess_image_format_from_filename(fn)

def image_format_name_from_code(code):
    return _plotstuff_c.image_format_name_from_code(code)

def parse_color(color, r, g, b, a):
    return _plotstuff_c.parse_color(color, r, g, b, a)

def parse_color_rgba(color, rgba):
    return _plotstuff_c.parse_color_rgba(color, rgba)

def cairo_set_color(cairo, color):
    return _plotstuff_c.cairo_set_color(cairo, color)

def cairo_set_rgba(cairo, rgba):
    return _plotstuff_c.cairo_set_rgba(cairo, rgba)

def plotstuff_new():
    return _plotstuff_c.plotstuff_new()

def plotstuff_init(plotargs):
    return _plotstuff_c.plotstuff_init(plotargs)

def plotstuff_read_and_run_command(pargs, f):
    return _plotstuff_c.plotstuff_read_and_run_command(pargs, f)

def plotstuff_run_command(pargs, cmd):
    return _plotstuff_c.plotstuff_run_command(pargs, cmd)

def plotstuff_set_text_bg_alpha(pargs, alpha):
    return _plotstuff_c.plotstuff_set_text_bg_alpha(pargs, alpha)

def plotstuff_plot_layer(pargs, layer):
    return _plotstuff_c.plotstuff_plot_layer(pargs, layer)

def plotstuff_get_config(pargs, name):
    return _plotstuff_c.plotstuff_get_config(pargs, name)

def plotstuff_set_color(pargs, name):
    return _plotstuff_c.plotstuff_set_color(pargs, name)

def plotstuff_set_bgcolor(pargs, name):
    return _plotstuff_c.plotstuff_set_bgcolor(pargs, name)

def plotstuff_get_alpha(pargs):
    return _plotstuff_c.plotstuff_get_alpha(pargs)

def plotstuff_set_alpha(pargs, alpha):
    return _plotstuff_c.plotstuff_set_alpha(pargs, alpha)

def plotstuff_set_rgba(pargs, rgba):
    return _plotstuff_c.plotstuff_set_rgba(pargs, rgba)

def plotstuff_set_rgba2(pargs, r, g, b, a):
    return _plotstuff_c.plotstuff_set_rgba2(pargs, r, g, b, a)

def plotstuff_set_bgrgba2(pargs, r, g, b, a):
    return _plotstuff_c.plotstuff_set_bgrgba2(pargs, r, g, b, a)

def plotstuff_set_marker(pargs, name):
    return _plotstuff_c.plotstuff_set_marker(pargs, name)

def plotstuff_set_markersize(pargs, ms):
    return _plotstuff_c.plotstuff_set_markersize(pargs, ms)

def plotstuff_set_size(pargs, W, H):
    return _plotstuff_c.plotstuff_set_size(pargs, W, H)

def plotstuff_set_size_wcs(pargs):
    return _plotstuff_c.plotstuff_set_size_wcs(pargs)

def plotstuff_scale_wcs(pargs, scale):
    return _plotstuff_c.plotstuff_scale_wcs(pargs, scale)

def plotstuff_rotate_wcs(pargs, angle):
    return _plotstuff_c.plotstuff_rotate_wcs(pargs, angle)

def plotstuff_set_wcs_box(pargs, ra, dec, width):
    return _plotstuff_c.plotstuff_set_wcs_box(pargs, ra, dec, width)

def plotstuff_set_wcs_file(pargs, fn, ext):
    return _plotstuff_c.plotstuff_set_wcs_file(pargs, fn, ext)

def plotstuff_set_wcs(pargs, wcs):
    return _plotstuff_c.plotstuff_set_wcs(pargs, wcs)

def plotstuff_set_wcs_tan(pargs, wcs):
    return _plotstuff_c.plotstuff_set_wcs_tan(pargs, wcs)

def plotstuff_set_wcs_sip(pargs, wcs):
    return _plotstuff_c.plotstuff_set_wcs_sip(pargs, wcs)

def plotstuff_builtin_apply(cairo, args):
    return _plotstuff_c.plotstuff_builtin_apply(cairo, args)

def plotstuff_marker_in_bounds(pargs, x, y):
    return _plotstuff_c.plotstuff_marker_in_bounds(pargs, x, y)

def plotstuff_run_commandf(*args):
    return _plotstuff_c.plotstuff_run_commandf(*args)

def plotstuff_output(pargs):
    return _plotstuff_c.plotstuff_output(pargs)

def plotstuff_free(pargs):
    return _plotstuff_c.plotstuff_free(pargs)

def plotstuff_clear(pargs):
    return _plotstuff_c.plotstuff_clear(pargs)

def plotstuff_stack_marker(pargs, x, y):
    return _plotstuff_c.plotstuff_stack_marker(pargs, x, y)

def plotstuff_stack_arrow(pargs, x, y, x2, y2):
    return _plotstuff_c.plotstuff_stack_arrow(pargs, x, y, x2, y2)

def plotstuff_stack_text(pargs, cairo, txt, px, py):
    return _plotstuff_c.plotstuff_stack_text(pargs, cairo, txt, px, py)

def plotstuff_plot_stack(pargs, cairo):
    return _plotstuff_c.plotstuff_plot_stack(pargs, cairo)

def plotstuff_get_maximum_rgba(pargs):
    return _plotstuff_c.plotstuff_get_maximum_rgba(pargs)

def plotstuff_pixel_scale(pargs):
    return _plotstuff_c.plotstuff_pixel_scale(pargs)

def plotstuff_radec2xy(pargs, ra, dec):
    return _plotstuff_c.plotstuff_radec2xy(pargs, ra, dec)

def plotstuff_xy2radec(pargs, x, y, pre):
    return _plotstuff_c.plotstuff_xy2radec(pargs, x, y, pre)

def plotstuff_get_radec_center_and_radius(pargs):
    return _plotstuff_c.plotstuff_get_radec_center_and_radius(pargs)

def plotstuff_get_radec_bounds(pargs, stepsize):
    return _plotstuff_c.plotstuff_get_radec_bounds(pargs, stepsize)

def plotstuff_radec_is_inside_image(pargs, ra, dec):
    return _plotstuff_c.plotstuff_radec_is_inside_image(pargs, ra, dec)

def plotstuff_line_constant_ra(pargs, ra, dec1, dec2, startwithmove):
    return _plotstuff_c.plotstuff_line_constant_ra(pargs, ra, dec1, dec2, startwithmove)

def plotstuff_line_constant_dec(pargs, dec, ra1, ra2):
    return _plotstuff_c.plotstuff_line_constant_dec(pargs, dec, ra1, ra2)

def plotstuff_line_constant_dec2(pargs, dec, ra1, ra2, stepra):
    return _plotstuff_c.plotstuff_line_constant_dec2(pargs, dec, ra1, ra2, stepra)

def plotstuff_text_xy(pargs, ra, dec, label):
    return _plotstuff_c.plotstuff_text_xy(pargs, ra, dec, label)

def plotstuff_text_radec(pargs, ra, dec, label):
    return _plotstuff_c.plotstuff_text_radec(pargs, ra, dec, label)

def plotstuff_move_to_radec(pargs, ra, dec):
    return _plotstuff_c.plotstuff_move_to_radec(pargs, ra, dec)

def plotstuff_line_to_radec(pargs, ra, dec):
    return _plotstuff_c.plotstuff_line_to_radec(pargs, ra, dec)

def plotstuff_close_path(pargs):
    return _plotstuff_c.plotstuff_close_path(pargs)

def plotstuff_stroke(pargs):
    return _plotstuff_c.plotstuff_stroke(pargs)

def plotstuff_fill(pargs):
    return _plotstuff_c.plotstuff_fill(pargs)

def plotstuff_stroke_preserve(pargs):
    return _plotstuff_c.plotstuff_stroke_preserve(pargs)

def plotstuff_fill_preserve(pargs):
    return _plotstuff_c.plotstuff_fill_preserve(pargs)

def plotstuff_move_to(pargs, x, y):
    return _plotstuff_c.plotstuff_move_to(pargs, x, y)

def plotstuff_line_to(pargs, x, y):
    return _plotstuff_c.plotstuff_line_to(pargs, x, y)

def plotstuff_marker(pargs, x, y):
    return _plotstuff_c.plotstuff_marker(pargs, x, y)

def plotstuff_marker_radec(pargs, ra, dec):
    return _plotstuff_c.plotstuff_marker_radec(pargs, ra, dec)

def plotstuff_append_doubles(str, lst):
    return _plotstuff_c.plotstuff_append_doubles(str, lst)

def plotstuff_set_dashed(pargs, dashlen):
    return _plotstuff_c.plotstuff_set_dashed(pargs, dashlen)

def plotstuff_set_solid(pargs):
    return _plotstuff_c.plotstuff_set_solid(pargs)
PTYPE_FLOAT = _plotstuff_c.PTYPE_FLOAT
PTYPE_INT = _plotstuff_c.PTYPE_INT
PTYPE_DOUBLE = _plotstuff_c.PTYPE_DOUBLE
PTYPE_UINT8 = _plotstuff_c.PTYPE_UINT8
PTYPE_INT16 = _plotstuff_c.PTYPE_INT16
BPP_8_UNSIGNED = _plotstuff_c.BPP_8_UNSIGNED
BPP_16_SIGNED = _plotstuff_c.BPP_16_SIGNED
BPP_32_SIGNED = _plotstuff_c.BPP_32_SIGNED
BPP_IEEE_FLOAT = _plotstuff_c.BPP_IEEE_FLOAT
BPP_IEEE_DOUBLE = _plotstuff_c.BPP_IEEE_DOUBLE
BPP_DEFAULT = _plotstuff_c.BPP_DEFAULT
class qfitsdumper(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    filename = property(_plotstuff_c.qfitsdumper_filename_get, _plotstuff_c.qfitsdumper_filename_set)
    npix = property(_plotstuff_c.qfitsdumper_npix_get, _plotstuff_c.qfitsdumper_npix_set)
    ptype = property(_plotstuff_c.qfitsdumper_ptype_get, _plotstuff_c.qfitsdumper_ptype_set)
    ibuf = property(_plotstuff_c.qfitsdumper_ibuf_get, _plotstuff_c.qfitsdumper_ibuf_set)
    fbuf = property(_plotstuff_c.qfitsdumper_fbuf_get, _plotstuff_c.qfitsdumper_fbuf_set)
    dbuf = property(_plotstuff_c.qfitsdumper_dbuf_get, _plotstuff_c.qfitsdumper_dbuf_set)
    vbuf = property(_plotstuff_c.qfitsdumper_vbuf_get, _plotstuff_c.qfitsdumper_vbuf_set)
    out_ptype = property(_plotstuff_c.qfitsdumper_out_ptype_get, _plotstuff_c.qfitsdumper_out_ptype_set)

    def __init__(self):
        _plotstuff_c.qfitsdumper_swiginit(self, _plotstuff_c.new_qfitsdumper())
    __swig_destroy__ = _plotstuff_c.delete_qfitsdumper

# Register qfitsdumper in _plotstuff_c:
_plotstuff_c.qfitsdumper_swigregister(qfitsdumper)


def qfits_pixdump(arg1):
    return _plotstuff_c.qfits_pixdump(arg1)

def convolve_get_gaussian_kernel_f(sigma, nsigma, k0, NK):
    return _plotstuff_c.convolve_get_gaussian_kernel_f(sigma, nsigma, k0, NK)

def convolve_separable_f(img, W, H, kernel, k0, NK, outimg, tempimg):
    return _plotstuff_c.convolve_separable_f(img, W, H, kernel, k0, NK, outimg, tempimg)

def convolve_separable_weighted_f(img, W, H, weight, kernel, k0, NK, outimg, tempimg):
    return _plotstuff_c.convolve_separable_weighted_f(img, W, H, weight, kernel, k0, NK, outimg, tempimg)

def c_image_numpy_view(data, nx, ny):
    return _plotstuff_c.c_image_numpy_view(data, nx, ny)

def qfits_load_image(fn, ext=1, plane=0, map=1, ptype=PTYPE_FLOAT):
    ld = qfitsloader()
    ld.filename = fn
    ld.xtnum = ext
    ld.pnum = plane
    ld.map = map
    ld.ptype = ptype
    if qfitsloader_init(ld):
        raise RuntimeError('qfitsloader_init(file "%s", ext %i) failed' % (fn, ext))
    if qfits_loadpix(ld):
        raise RuntimeError('qfits_loadpix(file "%s", ext %i) failed' % (fn, ext))

    class qfits_image(object):
        def __init__(self, pix, nx, ny, ld):
            self.pix = pix
            self.nx = nx
            self.ny = ny
            self.ld = ld
        def __del__(self):
            qfitsloader_free_buffer(self.ld)

    return qfits_image(ld.fbuf, ld.lx, ld.ly, ld)


def free(ptr):
    return _plotstuff_c.free(ptr)

def point_in_polygon(x, y, polygon):
    return _plotstuff_c.point_in_polygon(x, y, polygon)

def tan_vectors(pt, vec1, vec2):
    return _plotstuff_c.tan_vectors(pt, vec1, vec2)

def invert_2by2(A, Ainv):
    return _plotstuff_c.invert_2by2(A, Ainv)

def invert_2by2_arr(A, Ainv):
    return _plotstuff_c.invert_2by2_arr(A, Ainv)

def is_power_of_two(x):
    return _plotstuff_c.is_power_of_two(x)

def matrix_matrix_3(m1, m2, result):
    return _plotstuff_c.matrix_matrix_3(m1, m2, result)

def matrix_vector_3(m, v, result):
    return _plotstuff_c.matrix_vector_3(m, v, result)

def dot_product_3(v1, v2):
    return _plotstuff_c.dot_product_3(v1, v2)

def vector_length_3(v):
    return _plotstuff_c.vector_length_3(v)

def vector_length_squared_3(v):
    return _plotstuff_c.vector_length_squared_3(v)

def inverse_3by3(matrix):
    return _plotstuff_c.inverse_3by3(matrix)

def image_to_xyz(uu, vv, s, transform):
    return _plotstuff_c.image_to_xyz(uu, vv, s, transform)

def fit_transform(star, field, N, trans):
    return _plotstuff_c.fit_transform(star, field, N, trans)

def uniform_sample(low, high):
    return _plotstuff_c.uniform_sample(low, high)

def gaussian_sample(mean, stddev):
    return _plotstuff_c.gaussian_sample(mean, stddev)
EDGE_TRUNCATE = _plotstuff_c.EDGE_TRUNCATE
EDGE_AVERAGE = _plotstuff_c.EDGE_AVERAGE

def get_output_image_size(W, H, blocksize, edgehandling, outw, outh):
    return _plotstuff_c.get_output_image_size(W, H, blocksize, edgehandling, outw, outh)

def average_image_f(image, W, H, blocksize, edgehandling, output):
    return _plotstuff_c.average_image_f(image, W, H, blocksize, edgehandling, output)

def average_weighted_image_f(image, weight, W, H, blocksize, edgehandling, output, nilval):
    return _plotstuff_c.average_weighted_image_f(image, weight, W, H, blocksize, edgehandling, output, nilval)

def imax(a, b):
    return _plotstuff_c.imax(a, b)

def imin(a, b):
    return _plotstuff_c.imin(a, b)

def distsq_exceeds(d1, d2, D, limit):
    return _plotstuff_c.distsq_exceeds(d1, d2, D, limit)

def square(d):
    return _plotstuff_c.square(d)

def inrange(ra, ralow, rahigh):
    return _plotstuff_c.inrange(ra, ralow, rahigh)

def distsq(d1, d2, D):
    return _plotstuff_c.distsq(d1, d2, D)

def cross_product(v1, v2, cross):
    return _plotstuff_c.cross_product(v1, v2, cross)

def normalize(x, y, z):
    return _plotstuff_c.normalize(x, y, z)

def normalize_3(xyz):
    return _plotstuff_c.normalize_3(xyz)
class plotimage_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    fn = property(_plotstuff_c.plotimage_args_fn_get, _plotstuff_c.plotimage_args_fn_set)
    format = property(_plotstuff_c.plotimage_args_format_get, _plotstuff_c.plotimage_args_format_set)
    resample = property(_plotstuff_c.plotimage_args_resample_get, _plotstuff_c.plotimage_args_resample_set)
    downsample = property(_plotstuff_c.plotimage_args_downsample_get, _plotstuff_c.plotimage_args_downsample_set)
    arcsinh = property(_plotstuff_c.plotimage_args_arcsinh_get, _plotstuff_c.plotimage_args_arcsinh_set)
    rgbscale = property(_plotstuff_c.plotimage_args_rgbscale_get, _plotstuff_c.plotimage_args_rgbscale_set)
    alpha = property(_plotstuff_c.plotimage_args_alpha_get, _plotstuff_c.plotimage_args_alpha_set)
    wcs = property(_plotstuff_c.plotimage_args_wcs_get, _plotstuff_c.plotimage_args_wcs_set)
    gridsize = property(_plotstuff_c.plotimage_args_gridsize_get, _plotstuff_c.plotimage_args_gridsize_set)
    image_low = property(_plotstuff_c.plotimage_args_image_low_get, _plotstuff_c.plotimage_args_image_low_set)
    image_high = property(_plotstuff_c.plotimage_args_image_high_get, _plotstuff_c.plotimage_args_image_high_set)
    image_null = property(_plotstuff_c.plotimage_args_image_null_get, _plotstuff_c.plotimage_args_image_null_set)
    image_valid_low = property(_plotstuff_c.plotimage_args_image_valid_low_get, _plotstuff_c.plotimage_args_image_valid_low_set)
    image_valid_high = property(_plotstuff_c.plotimage_args_image_valid_high_get, _plotstuff_c.plotimage_args_image_valid_high_set)
    n_invalid_low = property(_plotstuff_c.plotimage_args_n_invalid_low_get, _plotstuff_c.plotimage_args_n_invalid_low_set)
    n_invalid_high = property(_plotstuff_c.plotimage_args_n_invalid_high_get, _plotstuff_c.plotimage_args_n_invalid_high_set)
    n_invalid_null = property(_plotstuff_c.plotimage_args_n_invalid_null_get, _plotstuff_c.plotimage_args_n_invalid_null_set)
    fitsext = property(_plotstuff_c.plotimage_args_fitsext_get, _plotstuff_c.plotimage_args_fitsext_set)
    fitsplane = property(_plotstuff_c.plotimage_args_fitsplane_get, _plotstuff_c.plotimage_args_fitsplane_set)
    auto_scale = property(_plotstuff_c.plotimage_args_auto_scale_get, _plotstuff_c.plotimage_args_auto_scale_set)
    img = property(_plotstuff_c.plotimage_args_img_get, _plotstuff_c.plotimage_args_img_set)
    W = property(_plotstuff_c.plotimage_args_W_get, _plotstuff_c.plotimage_args_W_set)
    H = property(_plotstuff_c.plotimage_args_H_get, _plotstuff_c.plotimage_args_H_set)

    def _set_image_from_numpy(self, arr):
        return _plotstuff_c.plotimage_args__set_image_from_numpy(self, arr)

    def set_wcs_file(self, fn, ext):
        return _plotstuff_c.plotimage_args_set_wcs_file(self, fn, ext)

    def set_file(self, fn):
        return _plotstuff_c.plotimage_args_set_file(self, fn)

    def set_rgbscale(self, r, g, b):
        return _plotstuff_c.plotimage_args_set_rgbscale(self, r, g, b)

    def get_image_width(self):
        return _plotstuff_c.plotimage_args_get_image_width(self)

    def get_image_height(self):
        return _plotstuff_c.plotimage_args_get_image_height(self)

    def __init__(self):
        _plotstuff_c.plotimage_args_swiginit(self, _plotstuff_c.new_plotimage_args())
    __swig_destroy__ = _plotstuff_c.delete_plotimage_args

# Register plotimage_args in _plotstuff_c:
_plotstuff_c.plotimage_args_swigregister(plotimage_args)


def plot_image_get(pargs):
    return _plotstuff_c.plot_image_get(pargs)

def plot_image_set_wcs(args, filename, ext):
    return _plotstuff_c.plot_image_set_wcs(args, filename, ext)

def plot_image_init(args):
    return _plotstuff_c.plot_image_init(args)

def plot_image_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_image_command(command, cmdargs, args, baton)

def plot_image_plot(command, cr, args, baton):
    return _plotstuff_c.plot_image_plot(command, cr, args, baton)

def plot_image_free(args, baton):
    return _plotstuff_c.plot_image_free(args, baton)

def plot_image_getsize(args, W, H):
    return _plotstuff_c.plot_image_getsize(args, W, H)

def plot_image_set_filename(args, fn):
    return _plotstuff_c.plot_image_set_filename(args, fn)

def plot_image_setsize(pargs, args):
    return _plotstuff_c.plot_image_setsize(pargs, args)

def plot_image_get_percentile(pargs, args, percentile):
    return _plotstuff_c.plot_image_get_percentile(pargs, args, percentile)

def plot_image_add_to_pixels(args, rgb):
    return _plotstuff_c.plot_image_add_to_pixels(args, rgb)

def plot_image_scale_float(args, fimg):
    return _plotstuff_c.plot_image_scale_float(args, fimg)

def plot_image_rgba_data(cairo, args):
    return _plotstuff_c.plot_image_rgba_data(cairo, args)

def plot_image_read(pargs, args):
    return _plotstuff_c.plot_image_read(pargs, args)

def plot_image_make_color_transparent(args, r, g, b):
    return _plotstuff_c.plot_image_make_color_transparent(args, r, g, b)

def plot_image_describe(p):
    return _plotstuff_c.plot_image_describe(p)
class plotoutline_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    wcs = property(_plotstuff_c.plotoutline_args_wcs_get, _plotstuff_c.plotoutline_args_wcs_set)
    stepsize = property(_plotstuff_c.plotoutline_args_stepsize_get, _plotstuff_c.plotoutline_args_stepsize_set)
    fill = property(_plotstuff_c.plotoutline_args_fill_get, _plotstuff_c.plotoutline_args_fill_set)

    def set_wcs_file(self, fn, ext):
        return _plotstuff_c.plotoutline_args_set_wcs_file(self, fn, ext)

    def set_wcs_size(self, W, H):
        return _plotstuff_c.plotoutline_args_set_wcs_size(self, W, H)

    def set_wcs(self, wcs):
        return _plotstuff_c.plotoutline_args_set_wcs(self, wcs)

    def __init__(self):
        _plotstuff_c.plotoutline_args_swiginit(self, _plotstuff_c.new_plotoutline_args())
    __swig_destroy__ = _plotstuff_c.delete_plotoutline_args

# Register plotoutline_args in _plotstuff_c:
_plotstuff_c.plotoutline_args_swigregister(plotoutline_args)


def plot_outline_get(pargs):
    return _plotstuff_c.plot_outline_get(pargs)

def plot_outline_init(args):
    return _plotstuff_c.plot_outline_init(args)

def plot_outline_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_outline_command(command, cmdargs, args, baton)

def plot_outline_plot(command, cr, args, baton):
    return _plotstuff_c.plot_outline_plot(command, cr, args, baton)

def plot_outline_free(args, baton):
    return _plotstuff_c.plot_outline_free(args, baton)

def plot_outline_set_wcs_file(args, filename, ext):
    return _plotstuff_c.plot_outline_set_wcs_file(args, filename, ext)

def plot_outline_set_wcs_size(args, W, H):
    return _plotstuff_c.plot_outline_set_wcs_size(args, W, H)

def plot_outline_set_wcs(args, wcs):
    return _plotstuff_c.plot_outline_set_wcs(args, wcs)

def plot_outline_set_tan_wcs(args, wcs):
    return _plotstuff_c.plot_outline_set_tan_wcs(args, wcs)

def plot_outline_set_fill(args, fill):
    return _plotstuff_c.plot_outline_set_fill(args, fill)

def plot_outline_describe(p):
    return _plotstuff_c.plot_outline_describe(p)
class plotgrid_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    dolabel = property(_plotstuff_c.plotgrid_args_dolabel_get, _plotstuff_c.plotgrid_args_dolabel_set)
    rastep = property(_plotstuff_c.plotgrid_args_rastep_get, _plotstuff_c.plotgrid_args_rastep_set)
    decstep = property(_plotstuff_c.plotgrid_args_decstep_get, _plotstuff_c.plotgrid_args_decstep_set)
    ralabelstep = property(_plotstuff_c.plotgrid_args_ralabelstep_get, _plotstuff_c.plotgrid_args_ralabelstep_set)
    declabelstep = property(_plotstuff_c.plotgrid_args_declabelstep_get, _plotstuff_c.plotgrid_args_declabelstep_set)
    ralabeldir = property(_plotstuff_c.plotgrid_args_ralabeldir_get, _plotstuff_c.plotgrid_args_ralabeldir_set)
    declabeldir = property(_plotstuff_c.plotgrid_args_declabeldir_get, _plotstuff_c.plotgrid_args_declabeldir_set)
    ralo = property(_plotstuff_c.plotgrid_args_ralo_get, _plotstuff_c.plotgrid_args_ralo_set)
    rahi = property(_plotstuff_c.plotgrid_args_rahi_get, _plotstuff_c.plotgrid_args_rahi_set)
    declo = property(_plotstuff_c.plotgrid_args_declo_get, _plotstuff_c.plotgrid_args_declo_set)
    dechi = property(_plotstuff_c.plotgrid_args_dechi_get, _plotstuff_c.plotgrid_args_dechi_set)
    raformat = property(_plotstuff_c.plotgrid_args_raformat_get, _plotstuff_c.plotgrid_args_raformat_set)
    decformat = property(_plotstuff_c.plotgrid_args_decformat_get, _plotstuff_c.plotgrid_args_decformat_set)

    def set_formats(self, raformat, decformat):
        return _plotstuff_c.plotgrid_args_set_formats(self, raformat, decformat)

    def __init__(self):
        _plotstuff_c.plotgrid_args_swiginit(self, _plotstuff_c.new_plotgrid_args())
    __swig_destroy__ = _plotstuff_c.delete_plotgrid_args

# Register plotgrid_args in _plotstuff_c:
_plotstuff_c.plotgrid_args_swigregister(plotgrid_args)


def plot_grid_get(pargs):
    return _plotstuff_c.plot_grid_get(pargs)

def plot_grid_init(args):
    return _plotstuff_c.plot_grid_init(args)

def plot_grid_set_formats(grid, raformat, decformat):
    return _plotstuff_c.plot_grid_set_formats(grid, raformat, decformat)

def plot_grid_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_grid_command(command, cmdargs, args, baton)

def plot_grid_plot(command, cr, args, baton):
    return _plotstuff_c.plot_grid_plot(command, cr, args, baton)

def plot_grid_free(args, baton):
    return _plotstuff_c.plot_grid_free(args, baton)
DIRECTION_DEFAULT = _plotstuff_c.DIRECTION_DEFAULT
DIRECTION_POS = _plotstuff_c.DIRECTION_POS
DIRECTION_NEG = _plotstuff_c.DIRECTION_NEG
DIRECTION_POSNEG = _plotstuff_c.DIRECTION_POSNEG
DIRECTION_NEGPOS = _plotstuff_c.DIRECTION_NEGPOS

def plot_grid_add_label(pargs, ra, dec, lval, format):
    return _plotstuff_c.plot_grid_add_label(pargs, ra, dec, lval, format)

def plot_grid_find_ra_label_location(pargs, ra, cdec, decmin, decmax, dirn):
    return _plotstuff_c.plot_grid_find_ra_label_location(pargs, ra, cdec, decmin, decmax, dirn)

def plot_grid_find_dec_label_location(pargs, dec, cra, ramin, ramax, dirn):
    return _plotstuff_c.plot_grid_find_dec_label_location(pargs, dec, cra, ramin, ramax, dirn)

def plot_grid_describe(p):
    return _plotstuff_c.plot_grid_describe(p)
class plotindex_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    indexes = property(_plotstuff_c.plotindex_args_indexes_get, _plotstuff_c.plotindex_args_indexes_set)
    qidxes = property(_plotstuff_c.plotindex_args_qidxes_get, _plotstuff_c.plotindex_args_qidxes_set)
    stars = property(_plotstuff_c.plotindex_args_stars_get, _plotstuff_c.plotindex_args_stars_set)
    quads = property(_plotstuff_c.plotindex_args_quads_get, _plotstuff_c.plotindex_args_quads_set)
    fill = property(_plotstuff_c.plotindex_args_fill_get, _plotstuff_c.plotindex_args_fill_set)

    def add_file(self, fn):
        return _plotstuff_c.plotindex_args_add_file(self, fn)

    def __init__(self):
        _plotstuff_c.plotindex_args_swiginit(self, _plotstuff_c.new_plotindex_args())
    __swig_destroy__ = _plotstuff_c.delete_plotindex_args

# Register plotindex_args in _plotstuff_c:
_plotstuff_c.plotindex_args_swigregister(plotindex_args)


def plot_quad_xy(cairo, quadxy, dimquads):
    return _plotstuff_c.plot_quad_xy(cairo, quadxy, dimquads)

def plot_index_plotquad(cairo, pargs, args, index, quadnum, DQ):
    return _plotstuff_c.plot_index_plotquad(cairo, pargs, args, index, quadnum, DQ)

def plot_index_get(pargs):
    return _plotstuff_c.plot_index_get(pargs)

def plot_index_add_file(args, fn):
    return _plotstuff_c.plot_index_add_file(args, fn)

def plot_index_add_qidx_file(args, fn):
    return _plotstuff_c.plot_index_add_qidx_file(args, fn)

def plot_index_init(args):
    return _plotstuff_c.plot_index_init(args)

def plot_index_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_index_command(command, cmdargs, args, baton)

def plot_index_plot(command, cr, args, baton):
    return _plotstuff_c.plot_index_plot(command, cr, args, baton)

def plot_index_free(args, baton):
    return _plotstuff_c.plot_index_free(args, baton)

def plot_index_describe(p):
    return _plotstuff_c.plot_index_describe(p)
class plotxy_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    fn = property(_plotstuff_c.plotxy_args_fn_get, _plotstuff_c.plotxy_args_fn_set)
    ext = property(_plotstuff_c.plotxy_args_ext_get, _plotstuff_c.plotxy_args_ext_set)
    xcol = property(_plotstuff_c.plotxy_args_xcol_get, _plotstuff_c.plotxy_args_xcol_set)
    ycol = property(_plotstuff_c.plotxy_args_ycol_get, _plotstuff_c.plotxy_args_ycol_set)
    xoff = property(_plotstuff_c.plotxy_args_xoff_get, _plotstuff_c.plotxy_args_xoff_set)
    yoff = property(_plotstuff_c.plotxy_args_yoff_get, _plotstuff_c.plotxy_args_yoff_set)
    firstobj = property(_plotstuff_c.plotxy_args_firstobj_get, _plotstuff_c.plotxy_args_firstobj_set)
    nobjs = property(_plotstuff_c.plotxy_args_nobjs_get, _plotstuff_c.plotxy_args_nobjs_set)
    scale = property(_plotstuff_c.plotxy_args_scale_get, _plotstuff_c.plotxy_args_scale_set)
    xyvals = property(_plotstuff_c.plotxy_args_xyvals_get, _plotstuff_c.plotxy_args_xyvals_set)
    wcs = property(_plotstuff_c.plotxy_args_wcs_get, _plotstuff_c.plotxy_args_wcs_set)

    def __init__(self):
        _plotstuff_c.plotxy_args_swiginit(self, _plotstuff_c.new_plotxy_args())
    __swig_destroy__ = _plotstuff_c.delete_plotxy_args

# Register plotxy_args in _plotstuff_c:
_plotstuff_c.plotxy_args_swigregister(plotxy_args)


def plot_xy_get(pargs):
    return _plotstuff_c.plot_xy_get(pargs)

def plot_xy_init(args):
    return _plotstuff_c.plot_xy_init(args)

def plot_xy_setsize(args, xyargs):
    return _plotstuff_c.plot_xy_setsize(args, xyargs)

def plot_xy_clear_list(args):
    return _plotstuff_c.plot_xy_clear_list(args)

def plot_xy_set_xcol(args, col):
    return _plotstuff_c.plot_xy_set_xcol(args, col)

def plot_xy_set_ycol(args, col):
    return _plotstuff_c.plot_xy_set_ycol(args, col)

def plot_xy_set_filename(args, fn):
    return _plotstuff_c.plot_xy_set_filename(args, fn)

def plot_xy_set_wcs_filename(args, fn, ext):
    return _plotstuff_c.plot_xy_set_wcs_filename(args, fn, ext)

def plot_xy_set_offsets(args, xo, yo):
    return _plotstuff_c.plot_xy_set_offsets(args, xo, yo)

def plot_xy_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_xy_command(command, cmdargs, args, baton)

def plot_xy_plot(command, cairo, plotargs, baton):
    return _plotstuff_c.plot_xy_plot(command, cairo, plotargs, baton)

def plot_xy_free(args, baton):
    return _plotstuff_c.plot_xy_free(args, baton)

def plot_xy_vals(args, x, y):
    return _plotstuff_c.plot_xy_vals(args, x, y)

def plot_xy_describe(p):
    return _plotstuff_c.plot_xy_describe(p)
class plotradec_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    fn = property(_plotstuff_c.plotradec_args_fn_get, _plotstuff_c.plotradec_args_fn_set)
    ext = property(_plotstuff_c.plotradec_args_ext_get, _plotstuff_c.plotradec_args_ext_set)
    racol = property(_plotstuff_c.plotradec_args_racol_get, _plotstuff_c.plotradec_args_racol_set)
    deccol = property(_plotstuff_c.plotradec_args_deccol_get, _plotstuff_c.plotradec_args_deccol_set)
    firstobj = property(_plotstuff_c.plotradec_args_firstobj_get, _plotstuff_c.plotradec_args_firstobj_set)
    nobjs = property(_plotstuff_c.plotradec_args_nobjs_get, _plotstuff_c.plotradec_args_nobjs_set)
    radecvals = property(_plotstuff_c.plotradec_args_radecvals_get, _plotstuff_c.plotradec_args_radecvals_set)

    def __init__(self):
        _plotstuff_c.plotradec_args_swiginit(self, _plotstuff_c.new_plotradec_args())
    __swig_destroy__ = _plotstuff_c.delete_plotradec_args

# Register plotradec_args in _plotstuff_c:
_plotstuff_c.plotradec_args_swigregister(plotradec_args)


def plot_radec_get(pargs):
    return _plotstuff_c.plot_radec_get(pargs)

def plot_radec_reset(args):
    return _plotstuff_c.plot_radec_reset(args)

def plot_radec_init(args):
    return _plotstuff_c.plot_radec_init(args)

def plot_radec_set_racol(args, col):
    return _plotstuff_c.plot_radec_set_racol(args, col)

def plot_radec_set_deccol(args, col):
    return _plotstuff_c.plot_radec_set_deccol(args, col)

def plot_radec_set_filename(args, fn):
    return _plotstuff_c.plot_radec_set_filename(args, fn)

def plot_radec_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_radec_command(command, cmdargs, args, baton)

def plot_radec_count_inbounds(pargs, args):
    return _plotstuff_c.plot_radec_count_inbounds(pargs, args)

def plot_radec_plot(command, cairo, plotargs, baton):
    return _plotstuff_c.plot_radec_plot(command, cairo, plotargs, baton)

def plot_radec_free(args, baton):
    return _plotstuff_c.plot_radec_free(args, baton)

def plot_radec_vals(args, ra, dec):
    return _plotstuff_c.plot_radec_vals(args, ra, dec)

def plot_radec_describe(p):
    return _plotstuff_c.plot_radec_describe(p)
class plotmatch_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    matches = property(_plotstuff_c.plotmatch_args_matches_get, _plotstuff_c.plotmatch_args_matches_set)

    def __init__(self):
        _plotstuff_c.plotmatch_args_swiginit(self, _plotstuff_c.new_plotmatch_args())
    __swig_destroy__ = _plotstuff_c.delete_plotmatch_args

# Register plotmatch_args in _plotstuff_c:
_plotstuff_c.plotmatch_args_swigregister(plotmatch_args)


def plot_match_get(pargs):
    return _plotstuff_c.plot_match_get(pargs)

def plot_match_add_match(args, mo):
    return _plotstuff_c.plot_match_add_match(args, mo)

def plot_match_set_filename(args, filename):
    return _plotstuff_c.plot_match_set_filename(args, filename)

def plot_match_init(args):
    return _plotstuff_c.plot_match_init(args)

def plot_match_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_match_command(command, cmdargs, args, baton)

def plot_match_plot(command, cr, args, baton):
    return _plotstuff_c.plot_match_plot(command, cr, args, baton)

def plot_match_free(args, baton):
    return _plotstuff_c.plot_match_free(args, baton)

def plot_match_describe(p):
    return _plotstuff_c.plot_match_describe(p)
class annotation_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    NGC = property(_plotstuff_c.annotation_args_NGC_get, _plotstuff_c.annotation_args_NGC_set)
    constellations = property(_plotstuff_c.annotation_args_constellations_get, _plotstuff_c.annotation_args_constellations_set)
    constellation_lines = property(_plotstuff_c.annotation_args_constellation_lines_get, _plotstuff_c.annotation_args_constellation_lines_set)
    constellation_markers = property(_plotstuff_c.annotation_args_constellation_markers_get, _plotstuff_c.annotation_args_constellation_markers_set)
    constellation_labels = property(_plotstuff_c.annotation_args_constellation_labels_get, _plotstuff_c.annotation_args_constellation_labels_set)
    constellation_labels_long = property(_plotstuff_c.annotation_args_constellation_labels_long_get, _plotstuff_c.annotation_args_constellation_labels_long_set)
    constellation_lines_offset = property(_plotstuff_c.annotation_args_constellation_lines_offset_get, _plotstuff_c.annotation_args_constellation_lines_offset_set)
    constellation_pastel = property(_plotstuff_c.annotation_args_constellation_pastel_get, _plotstuff_c.annotation_args_constellation_pastel_set)
    bright = property(_plotstuff_c.annotation_args_bright_get, _plotstuff_c.annotation_args_bright_set)
    bright_labels = property(_plotstuff_c.annotation_args_bright_labels_get, _plotstuff_c.annotation_args_bright_labels_set)
    bright_pastel = property(_plotstuff_c.annotation_args_bright_pastel_get, _plotstuff_c.annotation_args_bright_pastel_set)
    HD = property(_plotstuff_c.annotation_args_HD_get, _plotstuff_c.annotation_args_HD_set)
    HD_labels = property(_plotstuff_c.annotation_args_HD_labels_get, _plotstuff_c.annotation_args_HD_labels_set)
    ngc_fraction = property(_plotstuff_c.annotation_args_ngc_fraction_get, _plotstuff_c.annotation_args_ngc_fraction_set)
    targets = property(_plotstuff_c.annotation_args_targets_get, _plotstuff_c.annotation_args_targets_set)
    hd_catalog = property(_plotstuff_c.annotation_args_hd_catalog_get, _plotstuff_c.annotation_args_hd_catalog_set)

    def add_target(self, ra, dec, name):
        return _plotstuff_c.annotation_args_add_target(self, ra, dec, name)

    def add_named_target(self, name):
        return _plotstuff_c.annotation_args_add_named_target(self, name)

    def clear_targets(self):
        return _plotstuff_c.annotation_args_clear_targets(self)

    def __init__(self):
        _plotstuff_c.annotation_args_swiginit(self, _plotstuff_c.new_annotation_args())
    __swig_destroy__ = _plotstuff_c.delete_annotation_args

# Register annotation_args in _plotstuff_c:
_plotstuff_c.annotation_args_swigregister(annotation_args)


def plot_annotations_init(args):
    return _plotstuff_c.plot_annotations_init(args)

def plot_annotations_get(pargs):
    return _plotstuff_c.plot_annotations_get(pargs)

def plot_annotations_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_annotations_command(command, cmdargs, args, baton)

def plot_annotations_plot(command, cr, args, baton):
    return _plotstuff_c.plot_annotations_plot(command, cr, args, baton)

def plot_annotations_free(args, baton):
    return _plotstuff_c.plot_annotations_free(args, baton)

def plot_annotations_set_hd_catalog(ann, hdfn):
    return _plotstuff_c.plot_annotations_set_hd_catalog(ann, hdfn)

def plot_annotations_add_named_target(ann, target):
    return _plotstuff_c.plot_annotations_add_named_target(ann, target)

def plot_annotations_add_target(ann, ra, dec, name):
    return _plotstuff_c.plot_annotations_add_target(ann, ra, dec, name)

def plot_annotations_clear_targets(ann):
    return _plotstuff_c.plot_annotations_clear_targets(ann)

def plot_annotations_describe(p):
    return _plotstuff_c.plot_annotations_describe(p)
class plothealpix_args(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    nside = property(_plotstuff_c.plothealpix_args_nside_get, _plotstuff_c.plothealpix_args_nside_set)
    stepsize = property(_plotstuff_c.plothealpix_args_stepsize_get, _plotstuff_c.plothealpix_args_stepsize_set)

    def __init__(self):
        _plotstuff_c.plothealpix_args_swiginit(self, _plotstuff_c.new_plothealpix_args())
    __swig_destroy__ = _plotstuff_c.delete_plothealpix_args

# Register plothealpix_args in _plotstuff_c:
_plotstuff_c.plothealpix_args_swigregister(plothealpix_args)


def plot_healpix_get(pargs):
    return _plotstuff_c.plot_healpix_get(pargs)

def plot_healpix_init(args):
    return _plotstuff_c.plot_healpix_init(args)

def plot_healpix_command(command, cmdargs, args, baton):
    return _plotstuff_c.plot_healpix_command(command, cmdargs, args, baton)

def plot_healpix_plot(command, cr, args, baton):
    return _plotstuff_c.plot_healpix_plot(command, cr, args, baton)

def plot_healpix_free(args, baton):
    return _plotstuff_c.plot_healpix_free(args, baton)

def plot_healpix_describe(p):
    return _plotstuff_c.plot_healpix_describe(p)
CAIRO_OPERATOR_CLEAR = _plotstuff_c.CAIRO_OPERATOR_CLEAR
CAIRO_OPERATOR_SOURCE = _plotstuff_c.CAIRO_OPERATOR_SOURCE
CAIRO_OPERATOR_OVER = _plotstuff_c.CAIRO_OPERATOR_OVER
CAIRO_OPERATOR_IN = _plotstuff_c.CAIRO_OPERATOR_IN
CAIRO_OPERATOR_OUT = _plotstuff_c.CAIRO_OPERATOR_OUT
CAIRO_OPERATOR_ATOP = _plotstuff_c.CAIRO_OPERATOR_ATOP
CAIRO_OPERATOR_DEST = _plotstuff_c.CAIRO_OPERATOR_DEST
CAIRO_OPERATOR_DEST_OVER = _plotstuff_c.CAIRO_OPERATOR_DEST_OVER
CAIRO_OPERATOR_DEST_IN = _plotstuff_c.CAIRO_OPERATOR_DEST_IN
CAIRO_OPERATOR_DEST_OUT = _plotstuff_c.CAIRO_OPERATOR_DEST_OUT
CAIRO_OPERATOR_DEST_ATOP = _plotstuff_c.CAIRO_OPERATOR_DEST_ATOP
CAIRO_OPERATOR_XOR = _plotstuff_c.CAIRO_OPERATOR_XOR
CAIRO_OPERATOR_ADD = _plotstuff_c.CAIRO_OPERATOR_ADD
CAIRO_OPERATOR_SATURATE = _plotstuff_c.CAIRO_OPERATOR_SATURATE
CAIRO_OPERATOR_MULTIPLY = _plotstuff_c.CAIRO_OPERATOR_MULTIPLY
CAIRO_OPERATOR_SCREEN = _plotstuff_c.CAIRO_OPERATOR_SCREEN
CAIRO_OPERATOR_OVERLAY = _plotstuff_c.CAIRO_OPERATOR_OVERLAY
CAIRO_OPERATOR_DARKEN = _plotstuff_c.CAIRO_OPERATOR_DARKEN
CAIRO_OPERATOR_LIGHTEN = _plotstuff_c.CAIRO_OPERATOR_LIGHTEN
CAIRO_OPERATOR_COLOR_DODGE = _plotstuff_c.CAIRO_OPERATOR_COLOR_DODGE
CAIRO_OPERATOR_COLOR_BURN = _plotstuff_c.CAIRO_OPERATOR_COLOR_BURN
CAIRO_OPERATOR_HARD_LIGHT = _plotstuff_c.CAIRO_OPERATOR_HARD_LIGHT
CAIRO_OPERATOR_SOFT_LIGHT = _plotstuff_c.CAIRO_OPERATOR_SOFT_LIGHT
CAIRO_OPERATOR_DIFFERENCE = _plotstuff_c.CAIRO_OPERATOR_DIFFERENCE
CAIRO_OPERATOR_EXCLUSION = _plotstuff_c.CAIRO_OPERATOR_EXCLUSION
CAIRO_OPERATOR_HSL_HUE = _plotstuff_c.CAIRO_OPERATOR_HSL_HUE
CAIRO_OPERATOR_HSL_SATURATION = _plotstuff_c.CAIRO_OPERATOR_HSL_SATURATION
CAIRO_OPERATOR_HSL_COLOR = _plotstuff_c.CAIRO_OPERATOR_HSL_COLOR
CAIRO_OPERATOR_HSL_LUMINOSITY = _plotstuff_c.CAIRO_OPERATOR_HSL_LUMINOSITY

def image_debug(img, W, H):
    return _plotstuff_c.image_debug(img, W, H)

def image_add(img, W, H, val):
    return _plotstuff_c.image_add(img, W, H, val)

def image_weighted_smooth(img, W, H, weight, sigma):
    return _plotstuff_c.image_weighted_smooth(img, W, H, weight, sigma)

def plotoutline_setattr(self, name, val):
    if name == 'wcs_file':
        if type(val) is tuple:
            (fn,ext) = val
        else:
            fn = val
            ext = 0
        plot_outline_set_wcs_file(self, fn, ext)
        return
    self.__swig__setattr__(name, val)

plotoutline_args.__swig__setattr__ = plotoutline_args.__setattr__
plotoutline_args.__setattr__ = plotoutline_setattr


def plotimage_set_image_from_numpy(self, img):
    rtn = self._set_image_from_numpy(img)
    if rtn:
        raise RuntimeError('set_image_from_numpy() failed')
plotimage_args.set_image_from_numpy = plotimage_set_image_from_numpy






# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from __future__ import print_function
import matplotlib
matplotlib.use('Agg')
import os.path
from pylab import *
from numpy import *
# test_tweak 2>tt.py
import tt

def savefig(fn):
    from pylab import savefig as sf
    print('Saving', fn)
    sf(fn)

if __name__ == '__main__':
    #print 'me:', __file__
    #tt = os.path.join(os.path.dirname(__file__), 'test_tweak')

    for run in [2,3]:

        tanxy = getattr(tt, 'origxy_%i' % run)
        xy = getattr(tt, 'xy_%i' % run)
        noisyxy = getattr(tt, 'noisyxy_%i' % run)
        gridx = getattr(tt, 'gridx_%i' % run)
        gridy = getattr(tt, 'gridy_%i' % run)
        truesip_a = getattr(tt, 'truesip_a_%i' % run)
        truesip_b = getattr(tt, 'truesip_b_%i' % run)
        sip_a = getattr(tt, 'sip_a_%i' % run)
        sip_b = getattr(tt, 'sip_b_%i' % run)
        x0,y0 = tt.x0, tt.y0
        
        truedxy = xy - tanxy
        obsdxy  = noisyxy - tanxy

        xlo,xhi = -500, 2500
        ylo,yhi = -500, 2500

        X1 = linspace(xlo, xhi, 100)
        Y1 = gridy
        X1,Y1 = meshgrid(X1,Y1)
        X1 = X1.T
        Y1 = Y1.T
        X2 = gridx
        Y2 = linspace(ylo, yhi, 100)
        X2,Y2 = meshgrid(X2,Y2)
        truesipx_x = zeros_like(X1)
        truesipy_x = zeros_like(X1)
        truesipx_y = zeros_like(Y2)
        truesipy_y = zeros_like(Y2)
        for xo,yo,c in truesip_a:
            truesipx_y += c * (X2 - x0)**xo * (Y2 - y0)**yo
            truesipx_x += c * (X1 - x0)**xo * (Y1 - y0)**yo
        for xo,yo,c in truesip_b:
            truesipy_y += c * (X2 - x0)**xo * (Y2 - y0)**yo
            truesipy_x += c * (X1 - x0)**xo * (Y1 - y0)**yo

        x = xy[:,0]
        y = xy[:,1]
        truedx = truedxy[:,0]
        truedy = truedxy[:,1]
        obsdx = obsdxy[:,0]
        obsdy = obsdxy[:,1]

        for order in range(2,6):
            clf()

            sipx_x = zeros_like(X1)
            sipy_x = zeros_like(X1)
            sipx_y = zeros_like(Y2)
            sipy_y = zeros_like(Y2)
            for xo,yo,c in sip_a[order]:
                sipx_y += c * (X2 - x0)**xo * (Y2 - y0)**yo
                sipx_x += c * (X1 - x0)**xo * (Y1 - y0)**yo
            for xo,yo,c in sip_b[order]:
                sipy_y += c * (X2 - x0)**xo * (Y2 - y0)**yo
                sipy_x += c * (X1 - x0)**xo * (Y1 - y0)**yo

            subplot(2,2,1)
            plot(x, truedx, 'bs', mec='b', mfc='None')
            plot(x, obsdx, 'r.')
            plot(X1, -truesipx_x, 'b-', alpha=0.2)
            plot(X1, -sipx_x, 'r-', alpha=0.2)
            xlabel('x')
            ylabel('dx')
            xlim(xlo, xhi)

            subplot(2,2,2)
            plot(x, truedy, 'bs', mec='b', mfc='None')
            plot(x, obsdy, 'r.')
            plot(X1, -truesipy_x, 'b-', alpha=0.2)
            plot(X1, -sipy_x, 'r-', alpha=0.2)
            xlabel('x')
            ylabel('dy')
            xlim(xlo, xhi)

            subplot(2,2,3)
            plot(y, truedx, 'bs', mec='b', mfc='None')
            plot(y, obsdx, 'r.')
            plot(Y2, -truesipx_y, 'b-', alpha=0.2)
            plot(Y2, -sipx_y, 'r-', alpha=0.2)
            xlabel('y')
            ylabel('dx')
            xlim(xlo, xhi)

            subplot(2,2,4)
            plot(y, truedy, 'bs', mec='b', mfc='None')
            plot(y, obsdy, 'r.')
            plot(Y2, -truesipy_y, 'b-', alpha=0.2)
            plot(Y2, -sipy_y, 'r-', alpha=0.2)
            xlabel('y')
            ylabel('dy')
            xlim(xlo, xhi)

            savefig('tt%i-%i.png' % (run, order))
    
            clf()
            subplot(111)

            plot(tanxy[:,0], tanxy[:,1], 'b.')
            plot(noisyxy[:,0], noisyxy[:,1], 'r.')
            plot(xy[:,0], xy[:,1], 'bo', mec='b', mfc='None')

            if False:
                X3,Y3 = meshgrid(linspace(xlo, xhi, 11),
                                 linspace(ylo, yhi, 11))
                truesipx = X3
                truesipy = Y3
                for xo,yo,c in truesip_a:
                    truesipx += c * (X3 - x0)**xo * (Y3 - y0)**yo
                for xo,yo,c in truesip_b:
                    truesipy += c * (X3 - x0)**xo * (Y3 - y0)**yo
                sipx = X3
                sipy = Y3
                for xo,yo,c in sip_a[order]:
                    sipx += c * (X3 - x0)**xo * (Y3 - y0)**yo
                for xo,yo,c in sip_b[order]:
                    sipy += c * (X3 - x0)**xo * (Y3 - y0)**yo
                plot(truesipx, truesipy, 'bs', mec='b', mfc='None')
                plot(sipx, sipy, 'ms', mec='m', mfc='None')

            plot(X1, Y1, 'g-', alpha=0.25)
            plot(X2, Y2, 'g-', alpha=0.25)

            plot(truesipx_x + X1, truesipy_x + Y1, 'b-', alpha=0.25)
            plot(truesipx_y + X2, truesipy_y + Y2, 'b-', alpha=0.25)

            plot(sipx_x + X1, sipy_x + Y1, 'r-', alpha=0.25)
            plot(sipx_y + X2, sipy_y + Y2, 'r-', alpha=0.25)

            xlim(xlo,xhi)
            ylim(ylo,yhi)
            savefig('ttxy%i-%i.png' % (run,order))



# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from __future__ import print_function
import matplotlib
matplotlib.use('Agg')
import pylab as plt
import numpy as np

import sys
import os
from astrometry.sdss.dr9 import *
from astrometry.sdss import *
from astrometry.util.plotutils import *

from scipy.ndimage.filters import *

def test_vs_idl():
    run, camcol, field = 1752, 3, 163
    band ='r'
    bandnum = band_index(band)

    datadir = os.path.join(os.path.dirname(__file__), 'testdata')

    sdss = DR9(basedir=datadir)
    sdss.retrieve('psField', run, camcol, field)
    psfield = sdss.readPsField(run, camcol, field)

    ps = PlotSequence('klpsf')

    # These psf*.fits files were produced on the NYU system via:
    #
    #  psfield = mrdfits('psField-001752-3-0163.fit', 3)
    #  psfimage = sdss_psf_recon(psfield, 1000., 0.)
    #  mwrfits,psfimage,'psf1k0.fits',/CREATE
    #
    for x,y,fn in [(0.,0.,'psf00.fits'),
                   (0., 1000., 'psf01k.fits'),
                   (1000., 0., 'psf1k0.fits'),
                   (0., 2000., 'psf02k.fits'),
                   (2000., 0., 'psf2k0.fits'),
                   (600.,500.,'psf.fits')]:

        psf0 = pyfits.open(os.path.join(datadir, fn))[0].data

        # The IDL code adds 0.5 to the pixel coords
        psf = psfield.getPsfAtPoints(bandnum, x + 0.5, y + 0.5)

        psf = psf.astype(np.float32)

        def show(im):
            plt.imshow(np.log10(np.maximum(im, 1e-4)), interpolation='nearest', origin='lower')
        
        plt.clf()
        show(psf0)
        plt.gray()
        plt.colorbar()
        plt.title('IDL: %.1f, %.1f' % (x,y))
        ps.savefig()

        plt.clf()
        show(psf)
        plt.colorbar()
        plt.title('Me: %.1f, %.1f' % (x,y))
        ps.savefig()

        plt.clf()
        plt.imshow(psf - psf0, interpolation='nearest', origin='lower')
        plt.colorbar()
        plt.title('Diff: %.1f, %.1f' % (x,y))
        ps.savefig()

        diff = psf - psf0

        print('Diff:', diff.min(), diff.max())
        rms = np.sqrt(np.mean(diff**2))
        print('RMS:', rms)
        assert(np.all(np.abs(diff) < 5e-8))
        assert(rms < 2e-8)

def test_vs_stars():
    ps = PlotSequence('klpsf')

    #run, camcol, field, band = 4948, 6, 249, 'r'
    run, camcol, field, band = 94, 6, 11, 'r'
    test_vs_stars_on(run, camcol, field, band, ps)

    return
    run, camcol, field = 1752, 3, 163
    band ='r'
    test_vs_stars_on(run, camcol, field, band, ps)
    
    #756-z6-700
    run, camcol, field = 756, 6, 700
    band ='z'
    test_vs_stars_on(run, camcol, field, band, ps)

def test_vs_stars_on(run, camcol, field, band, ps):

    bandnum = band_index(band)
    datadir = os.path.join(os.path.dirname(__file__), 'testdata')
    sdss = DR9(basedir=datadir)

    sdss.retrieve('psField', run, camcol, field)
    psfield = sdss.readPsField(run, camcol, field)

    sdss.retrieve('frame', run, camcol, field, band)
    frame = sdss.readFrame(run, camcol, field, band)
    img = frame.image
    H,W = img.shape

    fn = sdss.retrieve('photoObj', run, camcol, field)
    T = fits_table(fn)
    T.mag  = T.get('psfmag')[:,bandnum]
    T.flux = T.get('psfflux')[:,bandnum]
    # !!
    T.x = T.colc[:,bandnum] - 0.5
    T.y = T.rowc[:,bandnum] - 0.5
    T.cut(T.prob_psf[:,bandnum] == 1)
    T.cut(T.nchild == 0)
    T.cut(T.parent == -1)
    T.cut(T.flux > 1.)
    print(len(T), 'after flux cut')
    #T.cut(T.flux > 10.)
    #print len(T), 'after flux cut'
    #T.cut(T.flux > 20.)
    #print len(T), 'after flux cut'
    T.cut(np.argsort(-T.flux)[:25])
    # margin
    m = 30
    T.cut((T.x >= m) * (T.x < (W-m)) * (T.y >= m) * (T.y < (H-m)))
    #T.cut(np.argsort(T.mag))
    T.cut(np.argsort(-np.abs(T.x - T.y)))
    
    print(len(T), 'PSF stars')
        
    #R,C = 5,5
    #plt.clf()

    eigenpsfs  = psfield.getEigenPsfs(bandnum)
    eigenpolys = psfield.getEigenPolynomials(bandnum)
    RR,CC = 2,2
    
    xx,yy = np.meshgrid(np.linspace(0, W, 12), np.linspace(0, H, 8))

    ima = dict(interpolation='nearest', origin='lower')
    
    plt.clf()
    mx = None
    for i,(psf,poly) in enumerate(zip(eigenpsfs, eigenpolys)):
        print()
        print('Eigen-PSF', i)
        XO,YO,C = poly
        kk = np.zeros_like(xx)
        for xo,yo,c in zip(XO,YO,C):
            dk = (xx ** xo) * (yy ** yo) * c
            #print 'xo,yo,c', xo,yo,c, '-->', dk
            kk += dk
        print('Max k:', kk.max(), 'min', kk.min())
        print('PSF range:', psf.min(), psf.max())
        print('Max effect:', max(np.abs(kk.min()), kk.max()) * max(np.abs(psf.min()), psf.max()))
        
        plt.subplot(RR,CC, i+1)
        plt.imshow(psf * kk.max(), **ima)
        if mx is None:
            mx = (psf * kk.max()).max()
        else:
            plt.clim(-mx * 0.05, mx * 0.05)
        plt.colorbar()
    ps.savefig()

    psfs = psfield.getPsfAtPoints(bandnum, xx,yy)
    #print 'PSFs:', psfs
    ph,pw = psfs[0].shape
    psfs = np.array(psfs).reshape(xx.shape + (ph,pw))
    print('PSFs shape:', psfs.shape)
    psfs = psfs[:,:,15:36,15:36]
    ny,nx,ph,pw = psfs.shape
    psfmos = np.concatenate([psfs[i,:,:,:] for i in range(ny)], axis=1)
    print('psfmos', psfmos.shape)
    psfmos = np.concatenate([psfmos[i,:,:] for i in range(nx)], axis=1)
    print('psfmos', psfmos.shape)
    plt.clf()
    plt.imshow(np.log10(np.maximum(psfmos + 1e-3, 1e-3)), **ima)
    ps.savefig()
    
    diffs = None
    rmses = None
    
    for i in range(len(T)):

        xx = T.x[i]
        yy = T.y[i]
        
        ix = int(np.round(xx))
        iy = int(np.round(yy))
        dx = xx - ix
        dy = yy - iy

        S = 25
        stamp = img[iy-S:iy+S+1, ix-S:ix+S+1]

        L = 6
        Lx = lanczos_filter(L, np.arange(-L, L+1) - dx)
        Ly = lanczos_filter(L, np.arange(-L, L+1) - dy)
        sx = correlate1d(stamp, Lx, axis=1, mode='constant')
        shim = correlate1d(sx,  Ly, axis=0, mode='constant')
        shim /= (Lx.sum() * Ly.sum())
        
        psf = psfield.getPsfAtPoints(bandnum, xx, yy)
        mod = psf / psf.sum() * T.flux[i]

        psf2 = psfield.getPsfAtPoints(bandnum, yy, xx)
        mod2 = psf2 / psf2.sum() * T.flux[i]

        psf3 = psfield.getPsfAtPoints(bandnum, xx, yy).T
        mod3 = psf3 / psf3.sum() * T.flux[i]

        mods = [mod, mod2, mod3]

        if diffs is None:
            diffs = [np.zeros_like(m) for m in mods]
            rmses = [np.zeros_like(m) for m in mods]
            
        for m,rms,diff in zip(mods, rmses, diffs):
            diff += (m - shim)
            rms +=  (m - shim)**2

        if i > 10:
            continue
        
        def show(im):
            plt.imshow(np.log10(np.maximum(1e-3, im + 1e-3)), vmin=-3, vmax=0, **ima)
            plt.hot()
            plt.colorbar()
        
        R,C = 3,4
        plt.clf()
        plt.subplot(R,C,1)
        show(shim)
        plt.subplot(R,C,2)
        show(mods[0])
        plt.subplot(R,C,2+C)
        plt.imshow(mods[0] - shim, vmin=-0.05, vmax=0.05, **ima)
        plt.hot()
        plt.colorbar()

        plt.subplot(R,C,3)
        show(mods[1])

        plt.subplot(R,C,3+C)
        plt.imshow(mods[1] - shim, vmin=-0.05, vmax=0.05, **ima)
        plt.hot()
        plt.colorbar()
        
        plt.subplot(R,C,4)
        show(mods[2])
        
        plt.subplot(R,C,4+C)
        plt.imshow(mods[2] - shim, vmin=-0.05, vmax=0.05, **ima)
        plt.hot()
        plt.colorbar()

        plt.subplot(R,C,3+2*C)
        plt.imshow(mods[1] - mods[0], vmin=-0.05, vmax=0.05, **ima)
        plt.hot()
        plt.colorbar()

        plt.subplot(R,C,4+2*C)
        plt.imshow(mods[2] - mods[0], vmin=-0.05, vmax=0.05, **ima)
        plt.hot()
        plt.colorbar()

        plt.suptitle('%.0f, %.0f' % (xx,yy))
        ps.savefig()

    for diff in diffs:
        diff /= len(T)

    rmses = [np.sqrt(rms / len(T)) for rms in rmses]

    print('rms median', np.median(rmses[0]), 'mean', np.mean(rmses[0]))

    r0,r1 = [np.percentile(rmses[0], p) for p in [10,90]]
    
    R,C = 2,len(diffs)
    plt.clf()

    for i,(d,r) in enumerate(zip(diffs, rmses)):
        plt.subplot(R,C, 1+i)
        plt.imshow(d, vmin=-0.05, vmax=0.05, **ima)
        plt.colorbar()
        plt.hot()

        plt.subplot(R,C, 1+i+C)
        plt.imshow(r, vmin=r0, vmax=r1, **ima)
        plt.colorbar()
        plt.hot()
    
    ps.savefig()

            
        
if __name__ == '__main__':
    #test_vs_idl()
    test_vs_stars()
    



#Script for automatically downloading UCAC3 catalogue
#Last revision: Dec 15, 2014
#Author: Denis Vida, denis.vida@gmail.com

# This file is part of the Astrometry.net suite.
# Copyright 2014 Denis Vida.

# The Astrometry.net suite is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.

# The Astrometry.net suite is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the Astrometry.net suite ; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

from __future__ import print_function

import urllib
import os

prefix = 'http://cdsarc.u-strasbg.fr/ftp/cats/aliases/U/UCAC3/UCAC3/'

for i in range(1, 361):
    name = 'z'+str(i).zfill(3)+'.bz2'
    url_name = prefix+name

    print('Downloading: '+url_name)
    ucac_file = urllib.URLopener()
    ucac_file.retrieve(url_name, name)



# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from __future__ import print_function
import sys

import matplotlib
matplotlib.use('Agg')

from math import pi
from pylab import *
from numpy import *
from numpy.random import *
try:
    import pyfits
except ImportError:
    try:
        from astropy.io import fits as pyfits
    except ImportError:
        raise ImportError("Cannot import either pyfits or astropy.io.fits")

# Given an image and an xylist (including estimated image sigma),
# look at a cutout around each source position, add noise, and recompute
# dcen3x3 to find the noisy peak.

def dcen3a(f0, f1, f2):
    s = 0.5 * (f2 - f0)
    d = 2. * f1 - (f0 + f2)
    if (d <= 1.e-10*f0):
        return None
    aa = f1 + 0.5 * s * s / d
    sod = s / d
    return sod * (1. + (4./3.) * (0.25 * d / aa) * (1. - 4. * sod * sod)) + 1.


def dcen3b(f0, f1, f2):
    a = 0.5 * (f2 - 2*f1 + f0)
    b = f1 - a - f0
    xc = -0.5 * b / a
    if not (0.0 < xc < 2.0):
        return None
    return xc

def dcen3(f0, f1, f2):
    return dcen3a(f0,f1,f2)

def dcen3x3(image):
    my0 = dcen3(image[0,0], image[1,0], image[2,0])
    my1 = dcen3(image[0,1], image[1,1], image[2,1])
    my2 = dcen3(image[0,2], image[1,2], image[2,2])

    mx0 = dcen3(image[0,0], image[0,1], image[0,2])
    mx1 = dcen3(image[1,0], image[1,1], image[1,2])
    mx2 = dcen3(image[2,0], image[2,1], image[2,2])

    if None in [ mx0, mx1, mx2, my0, my1, my2 ]:
        return None

    # x = (y-1) mx + bx
    bx = (mx0 + mx1 + mx2) / 3.
    mx = (mx2 - mx0) / 2.

    # y = (x-1) my + by
    by = (my0 + my1 + my2) / 3.
    my = (my2 - my0) / 2.;

    # find intersection
    xc = (mx * (by - my - 1.) + bx) / (1. + mx * my)
    yc = (xc - 1.) * my + by

    # check that we are in the box
    if not ((0.0 < xc < 2.0) and (0.0 < yc < 2.0)):
        return None

    return (xc,yc)


if __name__ == '__main__':
    imgfn = sys.argv[1]
    xyfn = sys.argv[2]
    
    print('FITS Image', imgfn)
    print('xylist', xyfn)

    p = pyfits.open(imgfn)
    I = p[0].data
    print('Image is', I.shape)

    p = pyfits.open(xyfn)
    xy = p[1].data
    x = xy.field('X')
    y = xy.field('Y')
    flux = xy.field('FLUX')
    print('Sources:', len(x))

    hdr = p[1].header
    sigma = hdr['ESTSIGMA']
    print('Estimated sigma', sigma)

    N = 100

    dx = []
    dy = []
    dx2 = []
    dy2 = []
    fluxes = []
    fluxes2 = []
    Nout = 0
    Nout2 = 0
    for i in range(len(x)):
        #print 'peak',i
        ix = round(x[i]) - 1
        iy = round(y[i]) - 1
        cutout = I[iy-1:iy+2, ix-1:ix+2]
        if cutout.shape != (3,3):
            print('cutout is', cutout.shape)
            continue
        #cutout5 = I[range(iy-2, iy+3, 2), range(ix-2, ix+3, 2)]
        cutout5 = array([ I[range(iy-2, iy+3, 2), ix-2],
                          I[range(iy-2, iy+3, 2), ix  ],
                          I[range(iy-2, iy+3, 2), ix+2], ])
                          
        if cutout5.shape != (3,3):
            print('cutout5 has shape', cutout5.shape)
            print('yrange', range(iy-2, iy+3, 2))
            print('xrange', range(ix-2, ix+3, 2))
            print('cutout5', cutout5)
            continue

        for j in xrange(N):
            noise = normal(0, sigma, size=(3,3))
            img = cutout + noise
            cen = dcen3x3(img)
            # original center:
            xx = x[i] - ix
            yy = y[i] - iy
            if cen is not None:
                (cx,cy) = cen
                dx.append(cx - xx)
                dy.append(cy - yy)
                fluxes.append(flux[i])
            else:
                Nout += 1
                noise5 = normal(0, sigma, size=(3,3))
                cen2 = dcen3x3(cutout5 + noise5)
                if cen2 is None:
                    Nout2 += 1
                    continue
                (cx2, cy2) = cen2
                cx = 1. + (cx2-1.) * 2.
                cy = 1. + (cy2-1.) * 2.
                dx2.append(cx - xx)
                dy2.append(cy - yy)
                fluxes2.append(flux[i])


    dx = array(dx)
    dy = array(dy)
    dx2 = array(dx2)
    dy2 = array(dy2)

    print('A total of', Nout,  'of', (N*len(x)), '(%i %%)' % int(round(Nout*100./float(N*len(x)))),  'peaks moved outside the 3x3 box.')
    print('A total of', Nout2, 'of', (N*len(x)), '(%i %%)' % int(round(Nout2*100./float(N*len(x)))), 'peaks moved outside the 5x5 box.')
    figure()
    clf()
    subplot(1,2,1)
    plot(dx, dy, 'r.')
    axis('equal')
    ylim(-3,3)
    a=axis()
    axhline(0)
    axvline(0)
    axis(a)
    title('3x3 centroid error')
    xlabel('distance (pixels)')
        
    subplot(1,2,2)
    plot(dx2, dy2, 'b.')
    axis('equal')
    ylim(-3,3)
    axhline(0)
    axvline(0)
    axis(a)
    title('5x5 centroid error')
    xlabel('distance (pixels)')
    savefig('dxdy.png', dpi=75)
    subplot(111)

    dist = sqrt(dx**2 + dy**2)
    dist2 = sqrt(dx2**2 + dy2**2)
    clf()
    subplot(2,1,1)
    (nb,bins,patches) = hist(dist, 40)
    xlim(0,bins[-1])
    title('3x3 centroid error distance')
    #xlabel('pixels')
    subplot(2,1,2)
    hist(dist2, bins=bins)
    xlim(0,bins[-1])
    title('5x5 centroid error distance')
    xlabel('pixels')
    savefig('dists.png', dpi=75)

    clf()

    dboth = hstack((dx,dy))
    sigd = std(dboth)
    md = mean(dboth)
    xx = arange(dboth.min(), dboth.max(), 0.01)
    yy = (len(dboth)*(xx.max()-xx.min())/40) / (sigd * sqrt(2. * pi)) * exp(-((xx-md)**2)/(2*sigd**2))

    subplot(2,1,1)
    (n,bins,patches) = hist(dboth, 40)
    plot(xx, yy, 'r-')
    title('3x3 centroid coordinate errors (std=%g)' % sigd)
    #xlabel('pixels')

    dboth2 = hstack((dx2,dy2))
    sigd2 = std(dboth2)
    md2 = mean(dboth2)
    if len(dboth2):
        xx2 = arange(dboth2.min(), dboth2.max(), 0.01)
        yy2 = (len(dboth2)*(xx2.max()-xx2.min())/40) / (sigd2 * sqrt(2. * pi)) * exp(-((xx2-md2)**2)/(2*sigd2**2))
    else:
        xx2 = []
        yy2 = []
    subplot(2,1,2)
    hist(dboth2, bins=bins)
    plot(xx2, yy2, 'r-')
    title('5x5 centroid coordinate errors (std=%g)' % sigd2)
    xlabel('pixels')
    savefig('dboth.png', dpi=75)

    clf()
    subplot(2,1,1)
    plot(fluxes, dist, 'r.')
    ylabel('pixel distance')
    #xlabel('flux')
    title('3x3 centroid')
    subplot(2,1,2)
    plot(fluxes2, dist2, 'b.')
    ylabel('pixel distance')
    xlabel('flux')
    title('5x5 centroid')
    savefig('fluxdist.png', dpi=75)
    
    




#! /usr/bin/env python3
# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from __future__ import print_function
from urllib2 import urlopen
from urllib import urlencode
from urlparse import urlparse, urljoin

import os.path

from numpy import *

from astrometry.util.file import *
from astrometry.util.usnob_get_image import *

from optparse import OptionParser


if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('-r', '--ra-low', '--ra-lo', '--ra-min',
                      dest='ralo', type=float, help='Minimum RA')
    parser.add_option('-R', '--ra-high', '--ra-hi', '--ra-max',
                      dest='rahi', type=float, help='Maximum RA')
    parser.add_option('-d', '--dec-low', '--dec-lo', '--dec-min',
                      dest='declo', type=float, help='Minimum Dec')
    parser.add_option('-D', '--dec-high', '--dec-hi', '--dec-max',
                      dest='dechi', type=float, help='Maximum Dec')
    parser.add_option('-p', '--prefix',
                      dest='prefix', help='Output file prefix')
    parser.add_option('-s', '--survey',
                      dest='survey', help='Grab only one USNOB survey: poss-i, poss-ii, ... (see http://www.nofs.navy.mil/data/fchpix/cfch.html')
    parser.add_option('-P', '--plate',
                      dest='plate', help='Grab only one USNOB plate: "se0161", for example')
    parser.add_option('-c', '--continue',
                      dest='cont', action='store_true', help='Continue a previously interrupted transfer')

    parser.set_defaults(prefix='usnob', survey=None, plate=None,
                        ralo=None, rahi=None, declo=None, dechi=None, cont=False)
    (opt, args) = parser.parse_args()

    if opt.ralo is None or opt.rahi is None or opt.declo is None or opt.dechi is None:
        parser.print_help()
        parser.error('RA,Dec lo,hi are required.')

    radecs = []

    decstep = 14./60.
    Dec = arange(opt.declo, opt.dechi+decstep, decstep)
    for dec in Dec:
        rastep = 14./60./cos(deg2rad(dec))
        RA  = arange(opt.ralo , opt.rahi +rastep , rastep)
        for ra in RA:
            radecs.append((ra,dec))
    radecs = array(radecs)

    # Retrieve them in order of distance from the center of the region...
    #dists = [distsq_between_radecs(r,d, (opt.ralo+opt.rahi)/2., (opt.declo+opt.dechi)/2.)
    #         for (r,d) in radecs]
    dists = distsq_between_radecs(radecs[:,0], radecs[:,1],
                                  (opt.ralo+opt.rahi)/2., (opt.declo+opt.dechi)/2.)
    order = argsort(dists)

    for (ra,dec) in radecs[order]:
        (jpeg,fits) = get_usnob_images(ra, dec, fits=True, survey=opt.survey, justurls=True)
        print('got jpeg urls:', jpeg)
        print('got fits urls:', fits)
        if opt.plate is None:
            keepjpeg = jpeg
            keepfits = fits
        else:
            keepjpeg = [u for u in jpeg if opt.plate in u]
            keepfits = [u for u in fits if opt.plate in u]
            print('keep jpeg urls:', keepjpeg)
            print('keep fits urls:', keepfits)
        base = opt.prefix + '-%.3f-%.3f-' % (ra,dec)
        for url in keepjpeg:
            # like "fchlwFxSl_so0194.000.jpg"
            urlfn = url.split('/')[-1]
            urlfn = urlfn.split('_')[-1]
            fn = base + urlfn
            if opt.cont and os.path.exists(fn):
                print('File', fn, 'exists.')
                continue
            print('retrieving', url, 'to', fn)
            res = urlopen(url)
            write_file(res.read(), fn)
        for url in keepfits:
            urlfn = url.split('/')[-1]
            urlfn = urlfn.split('_')[-1]
            fn = base + urlfn + '.fits'
            if opt.cont and os.path.exists(fn):
                print('File', fn, 'exists.')
                continue
            print('retrieving', url, 'to', fn)
            res = urlopen(url)
            write_file(res.read(), fn)
        



# Script for automatically running astrometry.net index building
# Last revision: Dec 20, 2014
# Author: Denis Vida, denis.vida@gmail.com

# This file is part of the Astrometry.net suite.
# Copyright 2014 Denis Vida.
# The Astrometry.net suite is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
# The Astrometry.net suite is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with the Astrometry.net suite ; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import print_function
import time
import os

#Determine the range of scales (default: from 5 to 7):
scale_range = [5, 7]

#Base file name
base_file_name = 'cut-'
extenstion = '.fits'

#Determine the file naming range, e.g. the output names will be cut-00.fits, cut-01.fits, ..., cut-11.fits
file_range = [0, 11]

#Run name (number to identify the indexing run, default is current date + scale)
run_name = time.strftime('%Y%m%d')


print('Running index generation with following parameters:')
print('\t Scale range: ', scale_range)
print('\t Base file name: ', base_file_name + '??' + extenstion)
print('\t File range: ', file_range)
print('\t Run name: ', run_name)

for file_no in range(file_range[0], file_range[1]+1):
    for scale in range(scale_range[0], scale_range[1]+1):
        file_no = str(file_no).zfill(2)
        scale = str(scale)

        base_command = 'build-astrometry-index -i '+base_file_name+file_no+extenstion+' -o index-ucac4-'+scale+'-'+file_no+'.fits -P '+scale+' -S MAG -H '+file_no+' -s 1 -I '+run_name+scale.zfill(2)
        
        print(base_command)

        #Run command
        os.system(base_command)



#! /usr/bin/env python3
# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE

# Used to trim down the "hpslit"-merged USNO-B files before
# building indices out of them.
from __future__ import print_function

import sys
from optparse import OptionParser

try:
    import pyfits
except ImportError:
    try:
        from astropy.io import fits as pyfits
    except ImportError:
        raise ImportError("Cannot import either pyfits or astropy.io.fits")
from numpy import *

from astrometry.util.fits import *
from astrometry.util.healpix import *
from astrometry.util.starutil_numpy import *
from astrometry.util.usnob_cuts import *


def trim(infn, outfn):
    print('Reading', infn)
    X = fits_table(infn, columns=[
        'num_detections', 'flags', 'an_diffraction_spike',
        'field_1', 'field_3', 'magnitude_1', 'magnitude_3',
        'field_0', 'field_2', 'magnitude_0', 'magnitude_2',
        'ra', 'dec',
        ])
    print('Read', len(X), 'sources')
    print('Applying cuts')
    I = usnob_apply_cuts(X)

    # drop now-unwanted columns
    for c in ['flags', 'an_diffraction_spike',
              'num_detections' ]:
        X.delete_column(c)
    X.cut(I)
    print('Kept', len(X), 'sources')
    del I

    print('Computing average mags')

    X.field_0 = X.field_0.astype(np.int16)
    X.field_1 = X.field_1.astype(np.int16)
    X.field_2 = X.field_2.astype(np.int16)
    X.field_3 = X.field_3.astype(np.int16)
    X.magnitude_0 = X.magnitude_0.astype(np.float32)
    X.magnitude_1 = X.magnitude_1.astype(np.float32)
    X.magnitude_2 = X.magnitude_2.astype(np.float32)
    X.magnitude_3 = X.magnitude_3.astype(np.float32)

    usnob_compute_average_mags(X)
    for c in [
        'field_1', 'field_3', 'magnitude_1', 'magnitude_3',
        'field_0', 'field_2', 'magnitude_0', 'magnitude_2']:
        X.delete_column(c)

    X.r_mag = X.r_mag.astype(np.float32)
    X.b_mag = X.b_mag.astype(np.float32)
    print('Writing output to', outfn)
    X.writeto(outfn)
    del X

if __name__ == '__main__':
    #for hp in range(12):

    if False:
        # fitscopy usnob-07.fits"[#row<100000000]" usnob-07-a.fits
        # fitscopy usnob-07.fits"[#row>=100000000]" usnob-07-b.fits
        infn = 'usnob-07-a.fits'
        outfn = 'usnob-trimmed-07-a.fits'
        trim(infn, outfn)
    if False:
        infn = 'usnob-07-b.fits'
        outfn = 'usnob-trimmed-07-b.fits'
        trim(infn, outfn)
        # cp usnob-trimmed-07-a.fits 07a.fits
        # tabmerge usnob-trimmed-07-b.fits+1 07a.fits+1
        # mv 07a.fits usnob-trimmed-07.fits
        
    if False:
        infn = 'usnob-10-a.fits'
        outfn = 'usnob-trimmed-10-a.fits'
        trim(infn, outfn)
    if True:
        infn = 'usnob-10-b.fits'
        outfn = 'usnob-trimmed-10-b.fits'
        trim(infn, outfn)

    #for hp in range(7,12):
    #for hp in range(8,12):
    for hp in range(11,12):
        infn = 'usnob-%02i.fits' % hp
        outfn = 'usnob-trimmed-%02i.fits' % hp
        trim(infn, outfn)


        



# Script for automatically downloading UCAC4 catalogue
# Last revision: Jan 08, 2015
# Author: Denis Vida, denis.vida@gmail.com

# This file is part of the Astrometry.net suite.
# Copyright 2014 Denis Vida.
# The Astrometry.net suite is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
# The Astrometry.net suite is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with the Astrometry.net suite ; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
from __future__ import print_function


# Start dectination (in degrees, e.g. +50.0)
start_dec = +90.0

# End declination (in degrees, e.g. -30.0)
end_dec = -90.0

try:
    from urllib.request import URLopener
except ImportError:
    from urllib import URLopener
import os
from shutil import copyfileobj
import bz2

# Retry failed downloads (how many times?)
No_retries = 5

# The URL prefix to the FTP repository
prefix = 'http://cdsarc.u-strasbg.fr/viz-bin/ftp-index?/ftp/cats/aliases/U/UCAC4/UCAC4/u4b/'

def Convert_Dec_to_Znum(start_dec, end_dec):
    """ Convert declination range to UCAC4 file number range. """
    z_file_constatnt = 180.0/900

    if start_dec < end_dec:
        temp = start_dec
        start_dec = end_dec
        end_dec = start_dec

    if start_dec > +90.0:
        start_dec = +90.0

    if end_dec < -90.0:
        end_dec = -90.0

    start_z = int(-(start_dec-90)/z_file_constatnt) + 1
    end_z = int(-(end_dec-90)/z_file_constatnt)

    return start_z, end_z

def Download_File(name):
    """ Download UCAC4 file. """

    url_name = prefix+name
    ucac_file = URLopener()
    ucac_file.retrieve(url_name, name)

    inp = open(name, 'rb')
    bz2_file = bz2.BZ2File(name+'.bz2', 'wb', compresslevel=1)
    copyfileobj(inp, bz2_file)
    inp.close()
    bz2_file.close()

    os.remove(name)

    return 0

start_z, end_z = Convert_Dec_to_Znum(start_dec, end_dec)

# Downloading the catalogue
fail_list = []
successes = 0
for i in range(start_z, end_z+1):
    name = 'z'+str(i).zfill(3)

    print('Downloading: '+name)
    try:
        Download_File(name)

        successes += 1
    except:
        fail_list.append(name)
        print('ERROR downloading file: ', name)

# Retry failed downloads
for i in range(No_retries):
    for name in fail_list:
        print('Retrying:', name)
        try:
            Download_File(name)
            successes += 1
            fail_list.pop(fail_list.index(name))
        except:
            print('Will retry', name, 'again...')


if len(fail_list) == 0:
    print('SUCCESS! All files downloaded successfully!')
elif successes > 0:
    print('WARNING! PARTIAL SUCCESS:')
    print(successes, 'files downloaded successfully,', len(fail_list), 'failed!')
    print('These files were NOT downloaded:', fail_list)
else:
    print('ERROR! ALL FILES FAILED TO DOWNLOAD!')
    print('Check your internet connection or try downloading later...')



# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from __future__ import print_function
from __future__ import absolute_import
import numpy as np
import time
from .util import *
from .starutil_numpy import *


tan = Tan(5., 10., 500.5, 500.5,
          1e-3, 1e-6, 1e-7, 1e-3, 1000., 1000.)

r1,d1 = tan.pixelxy2radec(0.5, 0.5)
r2,d2 = tan.pixelxy2radec(1000.5, 0.5)
r3,d3 = tan.pixelxy2radec(0.5, 1000.5)
r4,d4 = tan.pixelxy2radec(1000.5, 1000.5)
r5,d5 = tan.pixelxy2radec(500.5, 500.5)

print('1', r1,d1)
print('2', r2,d2)
print('3', r3,d3)
print('4', r4,d4)
print('5', r5,d5)

halftan = tan.scale(0.5)

print('Half TAN:', halftan)

ok,x1,y1 = halftan.radec2pixelxy(r1, d1)
ok,x2,y2 = halftan.radec2pixelxy(r2, d2)
ok,x3,y3 = halftan.radec2pixelxy(r3, d3)
ok,x4,y4 = halftan.radec2pixelxy(r4, d4)
ok,x5,y5 = halftan.radec2pixelxy(r5, d5)

print('half1', x1,y1)
print('half2', x2,y2)
print('half3', x3,y3)
print('half4', x4,y4)
print('half5', x5,y5)

print('------')

sip = Sip(Tan(5., 10., 500.5, 500.5,
              1e-3, 1e-6, 1e-7, 1e-3, 1000., 1000.))
sip.set_a_term(2, 0, 1e-5)
sip.set_a_term(2, 1, -2e-8)
sip.set_b_term(2, 1, 1e-8)
sip.set_b_term(0, 3, 3e-8)
sip.a_order = 2
sip.b_order = 4

print('Sip:', sip)

sip_ensure_inverse_polynomials(sip)

print('Sip:', sip)

print('A:')
print(np.array(sip.a).reshape((10,10))[:4,:4])
print('B:')
print(np.array(sip.b).reshape((10,10))[:4,:4])
print('AP:')
print(np.array(sip.ap).reshape((10,10))[:5,:5])
print('BP:')
print(np.array(sip.bp).reshape((10,10))[:5,:5])

from .plotSipDistortion import plotDistortion
import pylab as plt

plotDistortion(sip, 1000, 1000, 15, exaggerate=10.)
plt.savefig('sip1.png')

r1,d1 = sip.pixelxy2radec(0.5, 0.5)
r2,d2 = sip.pixelxy2radec(1000.5, 0.5)
r3,d3 = sip.pixelxy2radec(0.5, 1000.5)
r4,d4 = sip.pixelxy2radec(1000.5, 1000.5)
r5,d5 = sip.pixelxy2radec(500.5, 500.5)

xx = np.linspace(0.5, 1000.5, 100)
yy = [0.5, 500.5, 1000.5]
for iy,y in enumerate(yy):
    XD,YD,XU,YU = [],[],[],[]
    for x in xx:
        xd,yd = sip.get_distortion(x, y)
        xu,yu = sip.get_undistortion(xd, yd)
        XD.append(xd)
        YD.append(yd)
        XU.append(xu)
        YU.append(yu)
    XD = np.array(XD)
    YD = np.array(YD)
    XU = np.array(XU)
    YU = np.array(YU)

    plt.clf()
    plt.plot(xx, XD-xx, 'k-')
    plt.plot(xx, YD-y , 'k--')
    plt.plot(xx, XU-XD, 'r-')
    plt.plot(xx, YU-YD , 'r--')
    ex = 10.
    plt.plot(xx, ex * (XU - xx), 'b-')
    plt.plot(xx, ex * (YU - y), 'b--')
    plt.savefig('sip-dist%i.png' % iy)

print('1', r1,d1)
print('2', r2,d2)
print('3', r3,d3)
print('4', r4,d4)
print('5', r5,d5)

ok,x1,y1 = sip.radec2pixelxy(r1, d1)
ok,x2,y2 = sip.radec2pixelxy(r2, d2)
ok,x3,y3 = sip.radec2pixelxy(r3, d3)
ok,x4,y4 = sip.radec2pixelxy(r4, d4)
ok,x5,y5 = sip.radec2pixelxy(r5, d5)

print('inv1', x1,y1)
print('inv2', x2,y2)
print('inv3', x3,y3)
print('inv4', x4,y4)
print('inv5', x5,y5)

halfsip = sip.scale(0.5)

print('Half SIP:', halfsip)

print('A:')
print(np.array(halfsip.a).reshape((10,10))[:4,:4])
print('B:')
print(np.array(halfsip.b).reshape((10,10))[:4,:4])
print('AP:')
print(np.array(halfsip.ap).reshape((10,10))[:5,:5])
print('BP:')
print(np.array(halfsip.bp).reshape((10,10))[:5,:5])

plotDistortion(halfsip, 500, 500, 15, exaggerate=10.)
plt.savefig('sip2.png')

ok,x1,y1 = halfsip.radec2pixelxy(r1, d1)
ok,x2,y2 = halfsip.radec2pixelxy(r2, d2)
ok,x3,y3 = halfsip.radec2pixelxy(r3, d3)
ok,x4,y4 = halfsip.radec2pixelxy(r4, d4)
ok,x5,y5 = halfsip.radec2pixelxy(r5, d5)

print('half1', x1,y1)
print('half2', x2,y2)
print('half3', x3,y3)
print('half4', x4,y4)
print('half5', x5,y5)

rh1,dh1 = halfsip.pixelxy2radec(0.5, 0.5)
rh2,dh2 = halfsip.pixelxy2radec(500.5, 0.5)
rh3,dh3 = halfsip.pixelxy2radec(0.5, 500.5)
rh4,dh4 = halfsip.pixelxy2radec(500.5, 500.5)
rh5,dh5 = halfsip.pixelxy2radec(250.5, 250.5)

print('half1', rh1,dh1)
print('half2', rh2,dh2)
print('half3', rh3,dh3)
print('half4', rh4,dh4)
print('half5', rh5,dh5)

ok,x1,y1 = halfsip.radec2pixelxy(rh1, dh1)
ok,x2,y2 = halfsip.radec2pixelxy(rh2, dh2)
ok,x3,y3 = halfsip.radec2pixelxy(rh3, dh3)
ok,x4,y4 = halfsip.radec2pixelxy(rh4, dh4)
ok,x5,y5 = halfsip.radec2pixelxy(rh5, dh5)

print('hinv1', x1,y1)
print('hinv2', x2,y2)
print('hinv3', x3,y3)
print('hinv4', x4,y4)
print('hinv5', x5,y5)


import sys
sys.exit(0)



x = np.random.uniform(size=10000) * 10
y = np.random.uniform(size=10000) * 100
xlo,xhi = 0., 9.
ylo,yhi = 5., 105.
nx,ny = 10,12
H = np.zeros((ny,nx), np.int32)
an_hist2d(x, y, H, xlo, xhi, ylo, yhi)

print(H.sum())

H2,xe,ye = np.histogram2d(x, y, range=((xlo,xhi),(ylo,yhi)), bins=(nx,ny))

assert(np.all(H == H2.T))

x2 = np.append(x, np.array([xlo,xlo,xhi,xhi]))
y2 = np.append(y, np.array([ylo,yhi,ylo,yhi]))

Hb = np.zeros((ny,nx), np.int32)
an_hist2d(x2, y2, Hb, xlo, xhi, ylo, yhi)

H2b,xe,ye = np.histogram2d(x2, y2, range=((xlo,xhi),(ylo,yhi)), bins=(nx,ny))

assert(np.all(Hb == H2b.T))

an_hist2d(x, y, Hb, xlo, xhi, ylo, yhi)

assert(np.all(Hb == (H2b.T + H2.T)))


import sys
sys.exit(0)




sip = Sip('dec095705.01.p.w.wcs')
x = np.random.uniform(2000, size=100)
y = np.random.uniform(4000, size=100)
ra,dec = sip.pixelxy2radec(x, y)

xx = x + np.random.normal(scale=0.1, size=x.shape)
yy = y + np.random.normal(scale=0.1, size=y.shape)

xyz = radectoxyz(ra, dec)
xy = np.vstack((xx,yy)).T
print('xyz', xyz.shape)
print('xy', xy.shape)

tan = Tan('dec095705.01.p.w.wcs')

sip2 = fit_sip_wcs_2(xyz, xy, None, tan, 2,2)

print('Got', sip2)
print('Vs truth', sip)

sip2.write_to('sip2.wcs')

xy = np.vstack((x,y)).T
sip3 = fit_sip_wcs_2(xyz, xy, None, tan, 2,2)
sip3.write_to('sip3.wcs')

import sys
sys.exit(0)


X = np.random.uniform(1000., size=1001).astype(np.float32)

med = flat_median_f(X)
print('Median', med)
print('vs np ', np.median(X))

for pct in [0., 1., 10., 25., 50., 75., 99., 100.]:
    p1 = flat_percentile_f(X, pct)
    p2 = np.percentile(X, pct)
    print('Percentile', pct)
    print('  ', p1)
    print('vs', p2)


import fitsio
#X = fitsio.read('nasty.fits')
X = fitsio.read('dsky.fits')
assert(np.all(np.isfinite(X)))
print('med', np.median(X))
print('flat...')
f = flat_median_f(X)
print('flat', f)

for seed in range(42, 100):
    np.random.seed(seed)

    #X = np.random.normal(scale=10.0, size=(1016,1016)).astype(np.float32)
    X = np.random.normal(scale=10.0, size=(1015,1015)).astype(np.float32)
    # X = np.random.normal(scale=10.0, size=(10,10)).astype(np.float32)

    for i in range(3):
        t0 = time.clock()
        m = np.median(X)
        t1 = time.clock() - t0
        print('np.median:', t1)
    print('value:', m)

    I = np.argsort(X.ravel())
    m = X.flat[I[len(I)/2]]
    print('element[N/2] =', m)

    for i in range(3):
        t0 = time.clock()
        pym = flat_median_f(X)
        t1 = time.clock() - t0
        print('flat_median:', t1)
    print('value:', pym)
    assert(pym == m)
        
wcs = Tan()

wcs.crval = (1.,2.)
print('crval', wcs.crval)
(cr0,cr1) = wcs.crval

wcs.crpix = (50,100)
print('crpix', wcs.crpix)

wcs.crpix[0] = 500
print('crpix', wcs.crpix)

y = wcs.crpix[1]
wcs.crval[0] = 1.

wcs.cd = [1e-4,2e-4,-3e-4,4e-4]
print('cd', wcs.cd)

print('wcs:', wcs)

#wcs = tan_t()
wcs.pixel_scale()
xyz = wcs.pixelxy2xyz(0, 0)
print('xyz', xyz)
rd = wcs.pixelxy2radec(0, 0)
print('rd', rd)
xy = wcs.radec2pixelxy(rd[0], rd[1])
print('xy', xy)

X,Y = np.array([1,2,3]), np.array([4,5,6])
print('X,Y', X,Y)
R,D = wcs.pixelxy2radec(X, Y)
print('R,D', R,D)


wcs = Tan(0., 0., 0., 0., 1e-3, 0., 0., 1e-3, 100., 100.)

x1 = np.arange(3.)
x2 = np.arange(3)
x3 = x1[np.newaxis,:]
x4 = x1[:,np.newaxis]
x5 = x1.astype(np.float32)
x6 = x1.astype(np.dtype(np.float64).newbyteorder('S'))
x7 = 3.
x8 = np.array(3.)

import gc
gc.collect()
gc.set_debug(gc.DEBUG_LEAK)

for r,d in [(1., 2.),
            (1., x1),
            (1, x2),
            (1, x1),
            (x1, x1),
            (x1, x3),
            (x3, x4),
            (1., x5),
            (1., x6),
            (1., x7),
            (1., x8),
            ]:
    print()
    print('testing radec2pixelxy')
    print('  r', type(r), end=' ')
    if hasattr(r, 'dtype'):
       print(r.dtype, end=' ')
    print(r)

    print('  d', type(d), end=' ')
    if hasattr(d, 'dtype'):
        print(d.dtype, end=' ')
    print(d)
    
    ok,x,y = wcs.radec2pixelxy(r, d)
    print('ok,x,y =', ok,x,y)
    print('  x', type(x), end=' ')
    if hasattr(x, 'dtype'):
        print(x.dtype, end=' ')
    print(x)
    print('  y', type(y), end=' ')
    if hasattr(y, 'dtype'):
        print(y.dtype, end=' ')
    print(y)

    
gc.collect()

print('Garbage:', len(gc.garbage))
for x in gc.garbage:
    print('  ', x)
    



#! /usr/bin/env python3
# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from __future__ import print_function
import sys
import os
from optparse import OptionParser

# from util/addpath.py
if __name__ == '__main__':
    mydir = sys.path[0]
    andir = os.path.dirname(mydir)
    rootdir = os.path.dirname(andir)
    sys.path.insert(1, rootdir)

from astrometry.plot.plotstuff import *
from astrometry.util.fits import *

def match_kdtree_catalog(wcs, catfn):
    from astrometry.libkd.spherematch import tree_open, tree_close, tree_build_radec, tree_free, trees_match, tree_permute
    from astrometry.libkd import spherematch_c
    from astrometry.util.starutil_numpy import deg2dist, xyztoradec
    import numpy as np
    import sys

    rc,dc = wcs.get_center()
    rr = wcs.get_radius()
    kd = tree_open(catfn)
    kd2 = tree_build_radec(np.array([rc]), np.array([dc]))
    r = deg2dist(rr)
    I,J,nil = trees_match(kd, kd2, r, permuted=False)
    del kd2
    xyz = kd.get_data(I.astype(np.uint32))
    I = kd.permute(I)
    del kd
    tra,tdec = xyztoradec(xyz)
    return tra, tdec, I
    

def get_annotations(wcs, opt):
    # Objects to annotate:
    annobjs = []
    
    if opt.uzccat:
        # FIXME -- is this fast enough, or do we need to cut these
        # targets first?
        T = fits_table(opt.uzccat)
        for i in range(len(T)):
            if not wcs.is_inside(T.ra[i], T.dec[i]):
                continue
            annobjs.append((T.ra[i], T.dec[i], 'uzc', ['UZC %s' % T.zname[i]]))

    if opt.hipcat:
        # FIXME -- is this fast enough, or do we need to cut these
        # targets first?
        T = fits_table(opt.hipcat)
        for i in range(len(T)):
            if not wcs.is_inside(T.ra[i], T.dec[i]):
                continue
            if opt.hiplabel:
                txt = ['HIP %i (%.1f)' % (T.hip[i], T.vmag[i])]
            else:
                txt = ['']
            annobjs.append((T.ra[i], T.dec[i], 'HIP', txt))
            
    if opt.abellcat:
        T = fits_table(opt.abellcat)
        for i in range(len(T)):
            if not wcs.is_inside(T.ra[i], T.dec[i]):
                continue
            annobjs.append((T.ra[i], T.dec[i], 'abell', ['Abell %i' % T.aco[i]]))

    if opt.t2cat:
        #print 'Matching Tycho2...'
        tra,tdec,I2 = match_kdtree_catalog(wcs, opt.t2cat)
        T = fits_table(opt.t2cat, hdu=6)
        for r,d,t1,t2,t3 in zip(tra,tdec, T.tyc1[I2], T.tyc2[I2], T.tyc3[I2]):
            if not wcs.is_inside(r, d):
                continue
            annobjs.append((r, d, 'tycho2', ['Tycho-2 %i-%i-%i' % (t1,t2,t3)]))

    return annobjs


def get_annotations_for_wcs(wcs, opt):
    anns = get_annotations(wcs, opt)
    circs = []
    if opt.ngcnames:
        T = fits_table(opt.ngcnames)
        T.cut(np.array([len(s) for s in T.name]))
        T.isngc = np.array([not s.startswith('I') for s in T.name])
        T.num = np.array([int(s.replace('I','').strip()) for s in T.name])
        namemap = {}
        for X,nm in zip(zip(T.isngc, T.num), T.object):
            if not X in namemap:
                namemap[X] = []
            namemap[X].append(nm)
    else:
        namemap = None

    for nm,isngc,cat in [('NGC', True, opt.ngccat), ('IC', False, opt.iccat)]:
        if not cat:
            continue
        T = fits_table(cat)
        num = T.get(nm.lower() + 'num')
        for i in range(len(T)):
            # FIXME -- include NGC object radius...?
            if not wcs.is_inside(float(T.ra[i]), float(T.dec[i])):
                continue
            names = ['%s %i' % (nm, num[i])]
            if namemap:
                more = namemap.get((isngc, num[i]), None)
                if more:
                    names += more #' / '.join(more)
            if T.radius[i]:
                circs.append((T.ra[i], T.dec[i], nm.lower(), names, T.radius[i]))
            else:
                anns.append((T.ra[i], T.dec[i], nm.lower(), names))

    if opt.hdcat:
        #print 'Matching HD...'
        ra,dec,I = match_kdtree_catalog(wcs, opt.hdcat)
        for r,d,i in zip(ra,dec,I):
            if not wcs.is_inside(r, d):
                continue
            # good ol' HD catalog and its sensible numbering scheme
            anns.append((r, d, 'hd', ['HD %i' % (i+1)]))

    if opt.brightcat:
        T = fits_table(opt.brightcat)
        if opt.nbright:
            T.cut(np.argsort(T.vmag))
        nb = 0
        for r,d,n1,n2,vmag in zip(T.ra, T.dec, T.name1, T.name2, T.vmag):
            if not wcs.is_inside(r, d):
                continue
            # print('Bright-star catalog:', n1, n2, type(n1))
            names = []
            n1 = n1.strip()
            if len(n1):
                if '\\u' in n1:
                    try:
                        # py2 only
                        n1 = unicode(n1)
                    except:
                        pass
                    import json
                    x = json.dumps(n1)
                    x = x.replace('\\\\','\\')
                    n1 = json.loads(x)
                names.append(n1)
            n2 = n2.strip()
            if len(n2):
                names.append(n2)
            if len(names) == 0:
                # skip unnamed stars
                continue
            anns.append((r, d, 'bright', names, vmag))
            nb += 1
            if opt.nbright and nb >= opt.nbright:
                break
            
    jobjs = []
    for ann in anns:
        r,d,typ,names = ann[:4]
        ok,x,y = wcs.radec2pixelxy(float(r),float(d))
        dd = dict(type=typ, names=names, pixelx=x, pixely=y,
                  radius=0.)
        if len(ann) == 5:
            mag = ann[4]
            dd.update(vmag=mag)
        jobjs.append(dd)
    for r,d,typ,names,rad in circs:
        ok,x,y = wcs.radec2pixelxy(float(r),float(d))
        pixscale = wcs.pixel_scale()
        pixrad = (rad * 3600.) / pixscale
        jobjs.append(dict(type=typ, names=names, pixelx=x, pixely=y,
                          radius=pixrad))

    return jobjs

class OptDuck(object):
    pass

def get_empty_opts():
    opt = OptDuck()
    opt.ngc = False
    opt.bright = False
    opt.brightcat = None
    opt.nbright = 0
    opt.hdcat = None
    opt.uzccat = None
    opt.t2cat = None
    opt.abellcat = None
    opt.ngccat = None
    opt.ngcnames = None
    opt.iccat = None
    opt.hipcat = None
    opt.hiplabel = False
    return opt

if __name__ == '__main__':
    parser = OptionParser('usage: %prog <wcs.fits file> <image file> <output.{jpg,png,pdf} file>\n' +
                          '    OR     %prog <wcs.fits>   for JSON output')
    parser.add_option('--scale', dest='scale', type=float,
                      help='Scale plot by this factor')
    parser.add_option('--no-ngc', dest='ngc', action='store_false', default=True)
    parser.add_option('--no-bright', dest='bright', action='store_false', default=True)
    parser.add_option('--no-const', dest='const', action='store_false', default=True, help='Do not plot constellations')
    parser.add_option('--hdcat', dest='hdcat',
                      help='Path to Henry Draper catalog hd.fits')
    parser.add_option('--uzccat', dest='uzccat',
                      help='Path to Updated Zwicky Catalog uzc2000.fits')
    parser.add_option('--tycho2cat', dest='t2cat',
                      help='Path to Tycho-2 KD-tree file')
    parser.add_option('--abellcat', dest='abellcat',
                      help='Path to Abell catalog abell-all.fits')

    parser.add_option('--hipcat', dest='hipcat',
                      help='Path to Hipparcos catalog hip.fits')
    parser.add_option('--hiplabel', action='store_true',
                      help='Label Hipparcos stars')
    
    parser.add_option('--ngccat', dest='ngccat',
                      help='Path to NGC catalog openngc-ngc.fits -- ONLY USED FOR JSON OUTPUT!')
    parser.add_option('--ngcnames', dest='ngcnames',
                      help='Path to openngc-names.fits for aliases')
    parser.add_option('--iccat', dest='iccat',
                      help='Path to IC catalog openngc-ic.fits -- ONLY USED FOR JSON OUTPUT!')

    parser.add_option('--ngcfrac', dest='ngcfrac', type=float, default=0.,
                      help='Minimum fraction of image size to plot NGC/IC objs; default %default')

    parser.add_option('--brightcat', dest='brightcat',
                      help='Path to bright-star catalog -- ONLY USED FOR JSON OUTPUT!')

    parser.add_option('--nbright', dest='nbright', type=int, default=0,
                      help='Max number of bright stars')

    parser.add_option('--target', '-t', dest='target', action='append',
                      default=[],
                      help='Add named target (eg "M 31", "NGC 1499")')

    parser.add_option('--target-rd', '-T', dest='targetrd', nargs=3, action='append',
                      default=[],
                      help='Add a custom target with RA,Dec (eg, "-T \'My Star\' 34.33 0.87"')

    parser.add_option('--no-grid', dest='grid', action='store_false',
                      default=True, help='Turn off grid lines')
    parser.add_option('--grid-size', dest='gridsize', type=float,
                      default=0.1,
                      help='Grid spacing (in degrees), default %default')
    parser.add_option('--grid-color', dest='gridcolr',
                      default='0.2:0.2:0.2',
                      help='Grid color, default %default')
    parser.add_option('--grid-label', dest='gridlab', type=float,
                      default=0.2,
                      help='Grid label spacing (in degrees), default %default')

    parser.add_option('--tcolor', dest='textcolor', default='green',
                      help='Text color')
    parser.add_option('--tsize', dest='textsize', default=18, type=float,
                      help='Text font size')
    parser.add_option('--halign', dest='halign', default='C',
                      help='Text horizontal alignment')
    parser.add_option('--valign', dest='valign', default='B',
                      help='Text vertical alignment')
    parser.add_option('--tox', dest='tox', default=0, type=float,
                      help='Text offset x')
    parser.add_option('--toy', dest='toy', default=-10, type=float,
                      help='Text offset y')
    parser.add_option('--lw', dest='lw', default=2, type=float,
                      help='Annotations line width')
    parser.add_option('--ms', dest='ms', default=0., type=float,
                      help='Marker size')
    parser.add_option('--rd', dest='rd', action='append', default=[],
                      help='Plot RA,Dec markers')
    parser.add_option('--xy', dest='xy', action='append', default=[],
                      help='Plot x,y markers')
    parser.add_option('--quad', dest='quad', action='append', default=[],
                      help='Plot quad from given match file')
    parser.add_option('--pastel', action='store_true',
                      help='Pastel colors for constellations and bright stars?')
    parser.add_option('-v', '--verbose',
        action='store_true', dest='verbose', help='be chatty')

    opt,args = parser.parse_args()
    dojson = False
    if len(args) == 3:
        imgfn = args[1]
        outfn = args[2]
    else:
        if len(args) == 1:
            dojson = True
        else:
            parser.print_help()
            sys.exit(-1)

    import logging
    logformat = '%(message)s'
    from astrometry.util.util import log_init #, LOG_VERB, LOG_MSG
    loglvl = 2
    if opt.verbose:
        logging.basicConfig(level=logging.DEBUG, format=logformat)
        loglvl += 1
    else:
        logging.basicConfig(level=logging.INFO, format=logformat)
    log_init(loglvl)

    wcsfn = args[0]

    if dojson:
        from astrometry.util.util import anwcs
        wcs = anwcs(wcsfn,0)
        jobjs = get_annotations_for_wcs(wcs, opt)
        import json
        j = json.dumps(jobjs)
        print(j)
        sys.exit(0)

    fmt = PLOTSTUFF_FORMAT_JPG
    s = outfn.split('.')
    if len(s):
        s = s[-1].lower()
        if s in Plotstuff.format_map:
            fmt = s
    plot = Plotstuff(outformat=fmt, wcsfn=wcsfn)
    #plot.wcs_file = wcsfn
    #plot.outformat = fmt
    #plotstuff_set_size_wcs(plot.pargs)

    if opt.ms:
        plot.markersize = opt.ms
    plot.fontsize = opt.textsize

    plot.outfn = outfn
    img = plot.image
    img.set_file(imgfn)

    if opt.scale:
        plot.scale_wcs(opt.scale)
        plot.set_size_from_wcs()
        #W,H = img.get_size()

    plot.plot('image')

    if opt.grid:
        plot.color = opt.gridcolr
        plot.plot_grid(opt.gridsize, opt.gridsize,
                       opt.gridlab, opt.gridlab)

    ann = plot.annotations
    ann.NGC = opt.ngc
    ann.constellations = opt.const
    ann.constellation_labels = opt.const
    ann.constellation_labels_long = opt.const
    ann.bright = opt.bright
    ann.ngc_fraction = opt.ngcfrac
    if opt.pastel:
        ann.constellation_pastel = True
        ann.bright_pastel = True
        #print(ann.constellation_pastel)
    if opt.hdcat:
        ann.HD = True
        ann.HD_labels = True
        ann.hd_catalog = opt.hdcat

    anns = get_annotations(plot.wcs, opt)
    for r,d,typ,names in anns:
        name = ' / '.join(names)
        ann.add_target(r, d, name)

    plot.color = opt.textcolor
    plot.fontsize = opt.textsize
    plot.lw = opt.lw
    plot.valign = opt.valign
    plot.halign = opt.halign
    plot.label_offset_x = opt.tox;
    plot.label_offset_y = opt.toy;

    if len(opt.target):
        for t in opt.target:
            if plot_annotations_add_named_target(ann, t):
                raise RuntimeError('Unknown target', t)
    # if you want to plot normal vs named targets differently:
    # plot.plot('annotations')
    # ann.clear_targets()
    # ann.NGC = False
    # ann.constellations = False
    # ann.bright = False
    # ann.HD = False
    # plot.color = 'red'
    if len(opt.targetrd):
        for name,ra,dec in opt.targetrd:
            try:
                ra = float(ra)
            except:
                print('Failed to parse RA string as float:', ra)
                raise
            try:
                dec = float(dec)
            except:
                print('Failed to parse Dec string:', dec)
                raise
            ann.add_target(ra, dec, name)
    plot.plot('annotations')

    for rdfn in opt.rd:
        rd = plot.radec
        rd.fn = rdfn
        plot.plot('radec')

    for fn in opt.xy:
        xy = plot.xy
        xy.fn = fn
        plot.plot('xy')
        
    for mfn in opt.quad:
        match = fits_table(mfn)
        for m in match:
            qp = m.quadpix
            xy = [(qp[0], qp[1])]
            #plot.move_to_xy(qp[0], qp[1])
            for d in range(1, m.dimquads):
                #plot.line_to_xy(qp[2 * d], qp[2 * d + 1])
                xy.append((qp[2 * d], qp[2 * d + 1]))
            #plot.stroke()
            plot.polygon(xy)
            plot.close_path()
            plot.stroke()
        
    plot.write()



# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from astrometry.plot.plotstuff_c import *

# Could consider using swig's "addmethods" mechanism to create this "class" rep.

class Plotstuff(object):
    format_map = {'png': PLOTSTUFF_FORMAT_PNG,
                  'jpg': PLOTSTUFF_FORMAT_JPG,
                  'ppm': PLOTSTUFF_FORMAT_PPM,
                  'pdf': PLOTSTUFF_FORMAT_PDF,
                  'fits': PLOTSTUFF_FORMAT_FITS,
                  }


    def __init__(self, outformat=None, size=None, ra=None, dec=None, width=None,
                 rdw=None, wcsfn=None, wcsext=0, alpha=1., outfn=None):
        p = plotstuff_new()
        self.pargs = p
        if outformat is not None:
            outformat = Plotstuff.format_map.get(outformat, outformat)
            self.outformat = outformat
        if outfn is not None:
            self.outfn = outfn
        if size is not None:
            self.size = size
            self.color = 'black'
            self.alpha = alpha
            self.plot('fill')
        if ra is not None and dec is not None and width is not None:
            self.set_wcs_box(ra, dec, width)
        if rdw is not None:
            self.set_wcs_box(*rdw)
        if wcsfn is not None:
            self.wcs_file = (wcsfn, wcsext)
            if size is None:
                plotstuff_set_size_wcs(self.pargs)

    def __del__(self):
        #print 'plotstuff.__del__, pargs=', self.pargs
        plotstuff_free(self.pargs)

    def __getattr__(self, name):
        if name == 'xy':
            return plot_xy_get(self.pargs)
        elif name == 'index':
            return plot_index_get(self.pargs)
        elif name == 'radec':
            return plot_radec_get(self.pargs)
        elif name == 'match':
            return plot_match_get(self.pargs)
        elif name == 'image':
            return plot_image_get(self.pargs)
        elif name == 'outline':
            return plot_outline_get(self.pargs)
        elif name == 'grid':
            return plot_grid_get(self.pargs)
        elif name in ['ann', 'annotations']:
            return plot_annotations_get(self.pargs)
        elif name == 'healpix':
            return plot_healpix_get(self.pargs)
        return getattr(self.pargs, name)

    def __setattr__(self, name, val):
        if name == 'pargs':
            #print 'plotstuff.py: setting pargs to', val
            self.__dict__[name] = val
        elif name == 'size':
            #print 'plotstuff.py: setting plot size of', self.pargs, 'to %i,%i' % (val[0], val[1])
            plotstuff_set_size(self.pargs, val[0], val[1])
        elif name == 'color':
            #print 'plotstuff.py: setting color to "%s"' % val
            self.set_color(val)
        elif name == 'rgb':
            plotstuff_set_rgba2(self.pargs, val[0], val[1], val[2],
                           plotstuff_get_alpha(self.pargs))
        elif name == 'bg_rgba':
            plotstuff_set_bgrgba2(self.pargs, val[0], val[1], val[2], val[3])
        elif name == 'bg_box':
            self.pargs.bg_box = val
        elif name == 'rgba':
            plotstuff_set_rgba2(self.pargs, val[0], val[1], val[2], val[3])
        elif name == 'alpha':
            self.set_alpha(val)
        elif name == 'lw':
            self.pargs.lw = float(val)
        elif name == 'marker' and type(val) is str:
            plotstuff_set_marker(self.pargs, val)
        elif name == 'markersize':
            plotstuff_set_markersize(self.pargs, val)
        elif name == 'wcs_file':
            if type(val) is tuple:
                plotstuff_set_wcs_file(self.pargs, *val)
            else:
                plotstuff_set_wcs_file(self.pargs, val, 0)
        elif name == 'wcs':
            plotstuff_set_wcs(self.pargs, val)
        elif name == 'wcs_tan':
            plotstuff_set_wcs_tan(self.pargs, val)
        elif name == 'wcs_sip':
            plotstuff_set_wcs_sip(self.pargs, val)
        elif name == 'text_bg_alpha':
            plotstuff_set_text_bg_alpha(self.pargs, val)
        #elif name == 'operator':
        #    print 'val:', val
        #    self.pargs.op = val
        else:
            self.pargs.__setattr__(name, val)

    def line_constant_ra(self, ra, declo, dechi, startwithmove=True):
        return plotstuff_line_constant_ra(self.pargs, ra, declo, dechi, startwithmove)
    def line_constant_dec(self, dec, ralo, rahi):
        return plotstuff_line_constant_dec(self.pargs, dec, ralo, rahi)
    def line_constant_dec2(self, dec, ralo, rahi, rastep):
        return plotstuff_line_constant_dec2(self.pargs, dec, ralo, rahi, rastep)
    def fill(self):
        return plotstuff_fill(self.pargs)
        #return self.plot('fill')
    def stroke(self):
        return plotstuff_stroke(self.pargs)
    def fill_preserve(self):
        return plotstuff_fill_preserve(self.pargs)
    def stroke_preserve(self):
        return plotstuff_stroke_preserve(self.pargs)
    def move_to_radec(self, ra, dec):
        return plotstuff_move_to_radec(self.pargs, ra, dec)
    def line_to_radec(self, ra, dec):
        return plotstuff_line_to_radec(self.pargs, ra, dec)
    def move_to_xy(self, x, y):
        plotstuff_move_to(self.pargs, x, y)
    def line_to_xy(self, x, y):
        plotstuff_line_to(self.pargs, x, y)
    def close_path(self):
        return plotstuff_close_path(self.pargs)
    def dashed(self, dashlen):
        plotstuff_set_dashed(self.pargs, dashlen)
    def solid(self):
        plotstuff_set_solid(self.pargs)

    def set_size_from_wcs(self):
        self.pargs.set_size_from_wcs()

    def polygon(self, xy, makeConvex=True):
        import numpy as np
        if makeConvex:
            cx = sum(x for x,y in xy) / float(len(xy))
            cy = sum(y for x,y in xy) / float(len(xy))
            angles = np.array([np.arctan2(y - cy, x - cx)
                               for x,y in xy])
            I = np.argsort(angles)
        else:
            I = np.arange(len(xy))

        for j,i in enumerate(I):
            x,y = xy[i]
            if j == 0:
                self.move_to_xy(x, y)
            else:
                self.line_to_xy(x, y)

    def get_image_as_numpy(self, flip=False, out=None):
        # Caution: possible memory-handling problem with using "out"
        return self.pargs.get_image_as_numpy(flip, out)

    def get_image_as_numpy_view(self):
        return self.pargs.get_image_as_numpy_view()

    def set_image_from_numpy(self, img, flip=False):
        self.pargs.set_image_from_numpy(img, flip)

    def view_image_as_numpy(self):
        return self.pargs.view_image_as_numpy()

    def apply_settings(self):
        plotstuff_builtin_apply(self.pargs.cairo, self.pargs)

    def plot(self, layer):
        return plotstuff_plot_layer(self.pargs, layer)

    def scale_wcs(self, scale):
        plotstuff_scale_wcs(self.pargs, scale)

    def rotate_wcs(self, angle):
        plotstuff_rotate_wcs(self.pargs, angle)

    def set_wcs_box(self, ra, dec, width):
        plotstuff_set_wcs_box(self.pargs, ra, dec, width)

    def set_color(self, color):
        #print 'calling plotstuff_set_color(., \"%s\")' % color
        x = plotstuff_set_color(self.pargs, color)
        return x

    def set_alpha(self, a):
        x = plotstuff_set_alpha(self.pargs, a)

    def plot_grid(self, rastep, decstep, ralabelstep=None, declabelstep=None):
        import numpy as np

        grid = plot_grid_get(self.pargs)
        grid.rastep = rastep
        grid.decstep = decstep
        rformat = None
        if ralabelstep is None:
            ralabelstep = 0
        else:
            rdigits = max(0, np.ceil(-np.log10(ralabelstep)))
            rformat = '%.' + '%i'%rdigits + 'f'
        dformat = None
        if declabelstep is None:
            declabelstep = 0
        else:
            ddigits = max(0, np.ceil(-np.log10(declabelstep)))
            dformat = '%.' + '%i'%ddigits + 'f'
        if rformat is not None or dformat is not None:
            rformat = rformat or '%.2f'
            dformat = dformat or '%.2f'
            grid.set_formats(rformat, dformat)

        grid.ralabelstep = ralabelstep
        grid.declabelstep = declabelstep
        self.plot('grid')

    def clear(self):
        plotstuff_clear(self.pargs)

    def write(self, filename=None):
        if filename is not None:
            self.outfn = filename
        plotstuff_output(self.pargs)

    def text_xy(self, x, y, text):
        plotstuff_text_xy(self.pargs, x, y, text)

    def text_radec(self, ra, dec, text):
        plotstuff_text_radec(self.pargs, ra, dec, text)

    def stack_marker(self, x, y):
        plotstuff_stack_marker(self.pargs, x, y)

    def marker_xy(self, x,y):
        rtn = plotstuff_marker(self.pargs, x,y)

    def marker_radec(self, ra, dec):
        #print 'marker_radec', ra, dec
        rtn = plotstuff_marker_radec(self.pargs, ra, dec)
        #print '-> ', rtn
    def set_markersize(self, size):
        plotstuff_set_markersize(self.pargs, size)

    def plot_stack(self):
        plotstuff_plot_stack(self.pargs, self.pargs.cairo)



# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
from __future__ import print_function

"""
NAME:
      image2xy

PURPOSE:
      Extract sources from a FITS file

INPUTS:
      Takes a single FITS file as input

OPTIONAL INPUTS:

KEYWORD PARAMETERS:

OUTPUTS:

OPTIONAL OUTPUTS:

COMMENTS:

MODIFICATION HISTORY:
       K. Mierle, 2007-Jan - Initial version based on image2xy.c
       Hogg, 2007-May - simplexy options change
"""

# You need ctypes and a recent (1.0) numpy for this to work. I've included
# pyfits so you don't have to. 

try:
    import pyfits
except ImportError:
    try:
        from astropy.io import fits as pyfits
    except ImportError:
        raise ImportError("Cannot import either pyfits or astropy.io.fits")
import sys
import scipy
import os
import time
from simplexy import simplexy

# Default settings
dpsf = 1.0
plim = 8.0
dlim = dpsf
saddle = 5.0
maxper = 1000
maxnpeaks = 10000
maxsize = 1000
halfbox= 100

def source_extract(image_data, srcext=None):


    x,y,flux,sigma = simplexy(image_data, dpsf=dpsf, plim=plim,
                              dlim=dlim, saddle=saddle, maxper=maxper,
                              maxnpeaks=maxnpeaks, maxsize=maxsize,
                              halfbox=halfbox)

    print('simplexy: shapes',x.shape, y.shape)

    cx = pyfits.Column(name='X', format='E', array=x, unit='pix')
    cy = pyfits.Column(name='Y', format='E', array=y, unit='pix')
    cflux = pyfits.Column(name='FLUX', format='E', array=flux)

    tbhdu = pyfits.new_table([cx, cy, cflux])
    h = tbhdu.header
    h.add_comment('Parameters used in source extraction')
    h.update('dpsf',     dpsf, 'Gaussian psf width')
    h.update('plim',     plim, 'Significance to keep')
    h.update('dlim',     dlim, 'Closest two peaks can be')
    h.update('saddle',   saddle, 'Saddle in difference (in sig)')
    h.update('maxper',   maxper, 'Max num of peaks per object')
    h.update('maxpeaks', maxnpeaks, 'Max num of peaks total')
    h.update('maxsize',  maxsize, 'Max size of extended objects')
    h.update('halfbox',  halfbox, 'Half-size of sliding sky window')
    if srcext != None:
        h.update('srcext', i, 'Extension number in src image')
    h.update('estsigma', sigma, 'Estimated source image variance')
    h.add_comment('The X and Y points are specified assuming 1,1 is ')
    h.add_comment('the center of the leftmost bottom pixel of the ')
    h.add_comment('image in accordance with the FITS standard.')
    h.add_comment('Extracted by image2xy.py')
    h.add_comment('on %s %s' % (time.ctime(), time.tzname[0]))
    cards = tbhdu.header.ascardlist()
    cards['TTYPE1'].comment = 'X coordinate'
    cards['TTYPE2'].comment = 'Y coordinate'
    cards['TTYPE3'].comment = 'Flux of source'

    return tbhdu

def extract(fitsfile):
    outfile = pyfits.HDUList()

    # Make empty HDU; no image
    outfile.append(pyfits.PrimaryHDU()) 

    for i, hdu in enumerate(fitsfile):
        if (i == 0 and hdu.data != None) or isinstance(hdu, pyfits.ImageHDU):
            print(hdu.data.shape+(i,))
            if i == 0:
                print('Image: Primary HDU (number 0) %sx%s' % hdu.data.shape)
            else:
                print('Image: Extension HDU (number %s) %sx%s' % tuple((i,)+hdu.data.shape))

            tbhdu = source_extract(image_data)

            outfile.append(tbhdu)

    return x,y,flux,sigma, outfile

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: image2xy.py image.fits")
    else:
        infile = sys.argv[1]
        fitsfile = pyfits.open(infile)
        x,y,flux,sigma, outfile = extract(fitsfile)
        newfile = infile.replace('.fits','.xy.fits')
        try:
            outfile.writeto(newfile)
        except IOError:
            # File probably exists
            print('File %s appears to already exist; deleting!' % newfile)
            import os
            os.unlink(newfile)
            outfile.writeto(newfile)
